<?php
define('_FWFM_EXEC', true);

if (!class_exists('fwfmObject')) {
	class fwfmObject {
		var $_errors;
	
		function __construct() {
			$this->_errors = array();
		}
		function getProperties($public = true) {
			$vars = get_object_vars($this);
			if ($public) {
				foreach ($vars as $key => $value) {
					if ('_' == substr($key, 0, 1)) {
						unset($vars[$key]);
					}
				}
			}
	
			return $vars;
		}
	
		function setError($error) {
			$this->_errors[] = $error;
		}
		function getErrors() {
			return $this->_errors;
		}
		function getError() {
			if ($this->_errors) {
				return $this->_errors[count($this->_errors) - 1];
			}
		}
	}
}

if (!class_exists('fwfmDB')) {
	class fwfmDB extends fwfmObject {
		var $query;
		var $prefix;
		function setQuery($query, $limitstart=null, $limit=null) {
			global $wpdb;
			$this->prefix = $wpdb->prefix;
	
			if ($query == 'SELECT `sef` FROM `#__languages` WHERE `published` = 1 AND `ordering` = 1') {
				$query = 'SELECT \''.substr(get_locale(), 0, 2).'\' AS `sef`';
			} elseif (preg_match('#SELECT `lang_code` FROM `\#__languages`#i', $query)) {
				$lang = fwfmFactory::getLanguage();
				$query = 'SELECT \''.$lang->getTag().'\' AS `lang_code`';
			} elseif ($query == 'SELECT `element` FROM `#__extensions` WHERE `element` IN (\'codemirror\',\'none\') AND `folder` = \'editors\'') {
				$query = 'SELECT \'codemirror\' AS `element`';
			} elseif (strpos($query, '#__update') !== false) {
				$query = 'SELECT 1 AS `result`';
			}
	
			$this->query = str_replace('#__', $this->prefix, $query);
	
			if ($limitstart and $limit) {
				$this->query .= ' LIMIT '.$limitstart.', '.$limit;
			} elseif ($limitstart) {
				$this->query .= ' OFFSET '.$limitstart;
			} elseif ($limit) {
				$this->query .= ' LIMIT '.$limit;
			}
		}
		function getTableList() {
			global $wpdb;
			return $wpdb->get_col('SHOW TABLES');
		}
		function getPrefix() {
			return $this->prefix;
		}
		function getErrorMsg() {
			return $this->getError();
		}
	
		function getQuery() {
			return $this->query;
		}
		function getEscaped($value) {
			return esc_sql($value);
		}
		function quote($value) {
			return "'".esc_sql($value)."'";
		}
		function escape($value) {
			return esc_sql($value);
		}
		function insertid() {
			global $wpdb;
			return $wpdb->insert_id;
		}
		function query() {
			global $wpdb;
			if ($this->query) {
				$result = $wpdb->query($this->query);
				if ($result === false) {
					ob_start();
					$wpdb->print_error();
					$this->setError(ob_get_clean());
				} elseif ($result == 0) $result = true;
				return $result;
			}
		}
		function execute() {
			return $this->query();
		}
		function loadResult() {
			global $wpdb;
			$result = $wpdb->get_var($this->query, 0, 0);
			if ($result === false) {
				ob_start();
				$wpdb->print_error();
				$this->setError(ob_get_clean());
			}
			return $result;
		}
		function loadColumn() {
			global $wpdb;
			$result = $wpdb->get_col($this->query);
			if ($result === false) {
				ob_start();
				$wpdb->print_error();
				$this->setError(ob_get_clean());
			}
			return $result;
		}
		function loadAssoc() {
			global $wpdb;
			$result = $wpdb->get_row($this->query, ARRAY_A);
			if ($result === false) {
				ob_start();
				$wpdb->print_error();
				$this->setError(ob_get_clean());
			}
			return $result;
		}
		function loadObject() {
			global $wpdb;
			$result = $wpdb->get_row($this->query);
			if ($result === false) {
				ob_start();
				$wpdb->print_error();
				$this->setError(ob_get_clean());
			}
			return $result;
		}
		function loadAssocList($key=null) {
			global $wpdb;
			$result = $wpdb->get_results($this->query, ARRAY_A);
			if ($result === false) {
				ob_start();
				$wpdb->print_error();
				$this->setError(ob_get_clean());
			}
			return $result;
		}
		function loadObjectList($key=null) {
			global $wpdb;
			$result = $wpdb->get_results($this->query, OBJECT);
			if ($result === false) {
				ob_start();
				$wpdb->print_error();
				$this->setError(ob_get_clean());
			}
			return $result;
		}
	}
}

if (!class_exists('fwfmLayoutHelper')) {
	class fwfmLayoutHelper {
		static function render ($layout, $displayData=null, $path=null, $options=null) {
			if (is_null($path)) $path = FWFM_CONTROLLER_DIR.'layouts';
			elseif (!strpos($path, '/layouts')) $path .= '/layouts';
	
			$buff = explode('.', $layout);
			$filename = array_pop($buff).'.php';
			$layout_path = implode('/', $buff);
	
			if (file_exists($path.'/'.$layout_path.'/'.$filename)) {
				ob_start();
				include($path.'/'.$layout_path.'/'.$filename);
				return ob_get_clean();
			} else echo 'template '.$path.'/'.$layout_path.'/'.$filename.' not found';
		}
	}
}

if (!class_exists('fwfmDocument')) {
	class fwfmDocument {
		var $header = array();
		var $buttons = array();
		var $scripts = array();
		function addStyleDeclaration($style) {
			wp_add_inline_style(md5($style), $style);
		}
		function addScriptDeclaration($script, $type='') {
			if (!isset($this->scripts[$type])) {
				$this->scripts[$type] = array();
			}
			$md5 = md5($script);
			if (!isset($this->scripts[$type][$md5])) {
				$this->scripts[$type][$md5] = $script;
			}
		}
		function render($view) {
			ob_start();
			$view->display();
			$buff = ob_get_clean();
			if ($this->header or $this->buttons) {
?>
<div class="fwcss fwa-top-bar">
<?php
			if ($this->header && 0) {
?>
		<div class="col-lg-2 col-md-3 col-sm-4 fwa-title">
			<img class="fwa-logo" src="<?php echo fwfmURI::root(); ?>/admin/assets/images/icon_fw_food_menu_200.png" alt="FW Food Menu Logo">
			<h3><?php echo fwfmText::_('COM_FWFOODMENU'); ?></h3>
		</div>
<?php
				$this->header = array();
			}
			if ($this->buttons) {
?>
		<div class="col-lg-10 col-md-9 col-sm-8 fwa-top-buttons"><?php echo implode('', $this->buttons); ?></div>
<?php
				$this->buttons = array();
			}
?>
</div>
<?php
			}
			echo fwfmRoute::fwwpFixLinks($buff);
			if ($this->scripts) {
				foreach ($this->scripts as $type=>$script) {
?>
<script<?php if ($type) { ?> type="<?php echo $type; ?>"<?php } ?>><?php echo fwfmRoute::fwwpFixLinks(implode("\n", $script)); ?></script>
<?php
				}
				$this->scripts = array();
			}
		}
	}
}

if (!class_exists('fwfmRequest')) {
	class fwfmRequest {
		static function get($method='default') {
			$input = fwfmFactory::getApplication()->input;
			return $input->get($method);
		}
		static function setVar($name, $value) {
			$input = fwfmFactory::getApplication()->input;
			$input->set($name, $value);
		}
		static function getVar($name, $default=null, $method='default') {
			$input = fwfmFactory::getApplication()->input;
			return $input->getVar($name, $default, $method);
		}
		static function getCmd($name, $default='', $method='default') {
			$input = fwfmFactory::getApplication()->input;
			return $input->getCmd($name, $default, $method);
		}
		static function getString($name, $default='', $method='default') {
			$input = fwfmFactory::getApplication()->input;
			return $input->getString($name, $default, $method);
		}
		static function getInt($name, $default='', $method='default') {
			$input = fwfmFactory::getApplication()->input;
			return $input->getInt($name, $default, $method);
		}
	}
}

if (!class_exists('fwfmfwfmFilesInput')) {
	class fwfmfwfmFilesInput {
		function get($name) {
			$files = array();
			if (!empty($_FILES[$name]['name'])) {
				if (is_array($_FILES[$name]['name'])) {
					foreach ($_FILES[$name]['name'] as $i=>$filename) {
						$files[] = array(
							'name' => sanitize_file_name($filename),
							/* tmp_name is filled by php itself - so no need to sanitize/validate */
							'tmp_name' => $_FILES[$name]['tmp_name'][$i],
							'size' => (int)$_FILES[$name]['size'][$i],
							'error' => wp_kses($_FILES[$name]['error'][$i], wp_kses_allowed_html('strip'))
						);
					}
				} else {
					$files = array(
						'name' => sanitize_file_name($_FILES[$name]['name']),
						/* tmp_name is filled by php itself - so no need to sanitize/validate */
						'tmp_name' => $_FILES[$name]['tmp_name'],
						'size' => (int)$_FILES[$name]['size'],
						'error' => wp_kses($_FILES[$name]['error'], wp_kses_allowed_html('strip'))
					);
				}
			}
			return $files;
		}
	}
}
if (!class_exists('fwfmCookiesInput')) {
	class fwfmCookiesInput {
		function get($name, $def=null) {
			return wp_kses(fwfmArrayHelper::getValue($_COOKIE, $name, $def), wp_kses_allowed_html('post'));
		}
	}
}
if (!class_exists('fwfmInput')) {
	class fwfmInput {
		var $files = null,
			$cookie = null,
			$get = null,
			$post = null,
			$request = null,
			$methods = array('get', 'post', 'request');
		function __construct() {
			/* this class must have own copy of onchanged (sinitized, etc) request data */
			foreach ($this->methods as $method) {
				$sg = '_'.strtoupper($method);
				global $$sg;
				$this->$method = $$sg;
			}
			$this->files = new \fwfmfwfmFilesInput;
			$this->cookie = new \fwfmCookiesInput;
		}
		function set($name, $value) {
			foreach ($this->methods as $method) {
				$this->$method[$name] = $value;
			}
		}
		function get($name, $default='', $method='default') {
			switch ($method) {
				case 'get' :
					return fwfmArrayHelper::getValue($this->get, $name, $default);
					break;
				case 'post' :
					return fwfmArrayHelper::getValue($this->post, $name, $default);
					break;
				default :
					return fwfmArrayHelper::getValue($this->request, $name, $default);
			}
		}
		function getArray() {
			return $this->request;
		}
		function getRaw($name, $default='', $method='default') {
			switch ($method) {
				case 'get' :
					return fwfmArrayHelper::getValue($this->get, $name, $default);
					break;
				case 'post' :
					return fwfmArrayHelper::getValue($this->post, $name, $default);
					break;
				default :
					return fwfmArrayHelper::getValue($this->request, $name, $default);
			}
		}
		function getHtml($name, $default='', $method='default') {
			return wp_kses($this->getRaw($name, $default, $method), wp_kses_allowed_html('post'));
		}
		function getVar($name, $default='', $method='default') {
			return $this->getRaw($name, $default, $method);
		}
		function getCmd($name, $default='', $method='default') {
			return (string)wp_kses($this->getRaw($name, $default, $method), wp_kses_allowed_html('strip'));
		}
		function getString($name, $default='', $method='default') {
			return (string)wp_kses($this->getRaw($name, $default, $method), wp_kses_allowed_html('strip'));
		}
		function getInt($name, $default='', $method='default') {
			return (int)$this->getRaw($name, $default, $method);
		}
		function getFloat($name, $default='', $method='default') {
			return (float)$this->getRaw($name, $default, $method);
		}
	}
}

if (!class_exists('fwfmApplication')) {
	class fwfmApplication {
		var $messages = array();
		var $input;
		var $params;
		function __construct() {
			$this->input = new \fwfmInput;
			$this->params = fwfmComponentHelper::getParams('com_fwfoodmenu');
		}
		function triggerEvent($event, $params=null) {
			return fwfmDispatcher::trigger($event, $params);
		}
		function getUserStateFromRequest($key, $name, $def=null, $type=null) {
			$val = fwfmRequest::getVar($name);
			$key .= '.'.$name;
			if (!is_null($val)) $_SESSION[$key] = JFilterInput::sanitazeHtml($val);
			else $val = fwfmArrayHelper::getValue($_SESSION, $key);
			if (!$val) $val = $def;
	
			return $val;
		}
		function setUserState($name, $val) {
			$name = 'userstate.'.strtolower($name);
			$_SESSION[$name] = JFilterInput::sanitazeHtml($val);
		}
		function getUserState($name, $def=null) {
			$name = 'userstate.'.strtolower($name);
			$val = fwfmArrayHelper::getValue($_SESSION, $name);
			if (!$val) $val = $def;
	
			return $val;
		}
		function getCfg($name) {
			$result = '';
			switch ($name) {
				case 'list_limit' : $result = 20; break;
				case 'db' : $result = DB_NAME; break;
				case 'mailfrom' : $result = get_option('admin_email'); break;
				case 'fromname' : $result = get_option('blogname'); break;
			}
			return $result;
		}
		function isClient($client) {
			return ($client == 'administrator')?$this->isAdmin():$this->isSite();
		}
		function isAdmin() {
			$is_admin = is_admin();
	
			/** @todo make something not hardcoded to recognize site requests via wp-admin/admin-ajax.php */
			$name = fwfmRequest::getString('view');
			$format = fwfmRequest::getString('format', 'html');
			if ($is_admin and $name == 'menu' and $format == 'raw') {
				$is_admin = false;
			}
			return $is_admin;
		}
		function isSite() {
			return !$this->isAdmin();
		}
		function enqueueMessage($msg) {
			$this->messages[] = $msg;
			$this->setUserState('messages', $this->messages);
		}
		function redirect($uri) {
			$uri = str_replace('index.php?option=com_fwfoodmenu&view=', '?page=fwfm-', $uri);
			if (!headers_sent()) {
				header('location: '.$uri);
			}
?>
<script type="text/javascript">
location='<?php echo $uri; ?>';
</script>
<?php
			die();
		}
		function getParams() {
			return $this->params;
		}
	}
}

if (!class_exists('fwfmPagination')) {
	class fwfmPagination {
		var $total = 0,
			$limitstart = 0,
			$limit = 0,
			$link = '';
		function __construct($qty, $limitstart, $limit) {
			$app = fwfmFactory::getApplication();
			$this->total = $qty;
			$this->limitstart = $limitstart;
			$this->limit = $limit;
			$this->link = $app->isAdmin()?admin_url():site_url();
		}
		function setAdditionalUrlParam($key, $value) {
			$this->link .= (strpos($this->link, '?') === false?'?':'&').$key.'='.urlencode($value);
		}
		function orderUpIcon($i, $condition = true, $task = 'orderup', $alt = 'JLIB_HTML_MOVE_UP', $enabled = true, $checkbox = 'cb') {
			if (($i > 0 || ($i + $this->limitstart > 0)) && $condition) {
				return fwfmHtml::_('grid.orderUp', $i, $task, '', $alt, $enabled, $checkbox);
			} else {
				return '&#160;';
			}
		}
		function orderDownIcon($i, $n, $condition = true, $task = 'orderdown', $alt = 'JLIB_HTML_MOVE_DOWN', $enabled = true, $checkbox = 'cb') {
			if (($i < $n - 1 || $i + $this->limitstart < $this->total - 1) && $condition) {
				return fwfmHtml::_('grid.orderDown', $i, $task, '', $alt, $enabled, $checkbox);
			} else {
				return '&#160;';
			}
		}
		function getLimitBox() {
			ob_start();
?>
<div class="pull-right">
<?php
			echo fwfmHtml::_('select.genericlist', array(
				(object)array('id'=>'5','name'=>'5'),
				(object)array('id'=>'10','name'=>'10'),
				(object)array('id'=>'20','name'=>'20'),
				(object)array('id'=>'50','name'=>'50')
			), 'limit', 'class="form-comtrol" onchange="this.form.submit();"', 'id', 'name', $this->limit);
?>
</div>
<?php
			return ob_get_clean();
		}
		function getPagesLinks() {
			ob_start();
			if ($this->total > $this->limit) {
?>
<ul class="pagination mt-3">
<?php
				$j = 1;
				for ($i = 0; $i < $this->total; $i += $this->limit) {
					$active = ($i == $this->limitstart);
?>
	<li class="page-item<?php if ($active) { ?> active<?php } ?>">
<?php
					if ($active) {
?>
		<span class="page-link"><?php echo $j; ?></span>
<?php
					} else {
?>
		<a class="page-link" href="?page=<?php echo fwfmRequest::getString('page'); ?>&limitstart=<?php echo $i; ?>&limit=<?php echo $this->limit; ?>"><?php echo $j; ?></a>
<?php
					}
?>
	</li>
<?php
					$j++;
				}
?>
</ul>
<?php
			}
?>
<input type="hidden" name="limitstart" value="" />
<?php
			return ob_get_clean();
		}
		function getListFooter() {
//			echo $this->getLimitBox();
			echo $this->getPagesLinks();
		}
		public function getResultsCounter() {
			$html = null;
			$fromResult = $this->limitstart + 1;
	
			// If the limit is reached before the end of the list.
			if ($this->limitstart + $this->limit < $this->total) {
				$toResult = $this->limitstart + $this->limit;
			} else {
				$toResult = $this->total;
			}
	
			// If there are results found.
			if ($this->total > 0) {
				$msg = \fwfmText::sprintf('JLIB_HTML_RESULTS_OF', $fromResult, $toResult, $this->total);
				$html .= "\n" . $msg;
			} else {
				$html .= "\n" . \fwfmText::_('JLIB_HTML_NO_RECORDS_FOUND');
			}
	
			return $html;
		}
	}
}

if (!class_exists('fwfmToolBarHelper')) {
	class fwfmToolBarHelper {
		static function title($title) {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<h3><?php echo $title; ?></h3>
<?php
			$doc->header[] = ob_get_clean();
		}
		static function apply() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<button class="btn btn-success btn-apply" onclick="Joomla.submitbutton('apply');"><i class="far fa-save mr-1"></i> <?php echo fwfmText::_('Save'); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function save() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<button class="btn btn-save" onclick="Joomla.submitbutton('save');"><i class="fas fa-check mr-1 text-success"></i> <?php echo fwfmText::_('Save & Close'); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function save2New() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<button class="btn btn-save2new" onclick="Joomla.submitbutton('save2new');"><i class="fas fa-plus mr-1 text-success"></i> <?php echo fwfmText::_('Save & New'); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function save2Copy() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<button class="btn btn-save2copy" onclick="Joomla.submitbutton('save2copy');"><i class="far fa-copy mr-1 text-success"></i> <?php echo fwfmText::_('Save as Copy'); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function cancel() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<button class="btn btn-cancel" onclick="Joomla.submitbutton('cancel');"><i class="fas fa-times-circle mr-1 text-danger"></i> <?php echo fwfmText::_('Cancel'); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function addNew() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<button class="btn btn-success btn-add" onclick="Joomla.submitbutton('add');"><i class="fas fa-plus-circle mr-1"></i> <?php echo fwfmText::_('Add'); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function editList() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<button class="btn btn-edit" onclick="Joomla.submitbutton('edit');"><i class="far fa-edit mr-1"></i> <?php echo fwfmText::_('Edit'); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function deleteList() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<button class="btn btn-danger btn-remove" onclick="Joomla.submitbutton('remove');"><i class="far fa-trash-alt mr-1"></i> <?php echo fwfmText::_('Remove'); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function publishList() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<button class="btn btn-publish" onclick="Joomla.submitbutton('publish');"><i class="fas fa-check mr-1 text-success"></i> <?php echo fwfmText::_('Publish'); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function publish() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<button class="btn btn-publish" onclick="Joomla.submitbutton('publish');"><i class="fas fa-check mr-1 text-success"></i> <?php echo fwfmText::_('Publish'); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function unpublishList() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<button class="btn btn-unpublish" onclick="Joomla.submitbutton('unpublish');"><i class="fas fa-times mr-1 text-danger"></i> <?php echo fwfmText::_('Unpublish'); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function unpublish() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
<button class="btn btn-unpublish" onclick="Joomla.submitbutton('unpublish');"><i class="fas fa-times mr-1 text-danger"></i> <?php echo fwfmText::_('Unpublish'); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function divider() {
			$doc = fwfmFactory::getDocument();
			ob_start();
?>
|
<?php
			$doc->buttons[] = ob_get_clean();
		}
		static function custom($task, $image1, $image2, $title) {
			$doc = fwfmFactory::getDocument();
			switch ($image1) {
				case 'checkbox-partial' : $image1 = 'bars'; break;
			}
			ob_start();
?>
<button class="btn btn-<?php echo $task ?>" onclick="Joomla.submitbutton('<?php echo $task; ?>');"><i class="fal fa-<?php echo $image1; ?> mr-1"></i> <?php echo fwfmText::_($title); ?></button>
<?php
			$doc->buttons[] = ob_get_clean();
		}
	}
}

if (!class_exists('fwfmHtml')) {
	class fwfmHtml {
		static function _($name) {
			static $funcs = array();
			$name = strtolower($name);
			$buff = explode('.', $name, 2);
			if (count($buff) != 2) return;
			$class_name = 'fwfmHtml'.$buff[0];
			$method_name = $buff[1];
	
			if (!isset($funcs[$name])) {
				$paths = fwfmHtml::addIncludePath();
	
				foreach ($paths as $path) {
					$path .= $buff[0].'.php';
					if (file_exists($path)) {
						require_once($path);
						if (class_exists($class_name)) {
							$class_name = '\\'.$class_name;
							$funcs[$name] = new $class_name;
						}
						break;
					}
				}
			}
			if (isset($funcs[$name])) {
				if (method_exists($funcs[$name], $method_name)) {
					$args = func_get_args();
					array_shift($args);
					$temp = array();
					foreach ($args as &$arg) $temp[] = &$arg;
					return call_user_func_array(array($funcs[$name], $method_name), $temp);
				} else die('method "'.$class_name.':'.$method_name.'" not found');
			} else die('class "'.$class_name.'" not found');
		}
		static function addIncludePath($path=null) {
			static $paths = array();
			if (empty($paths)) {
				$paths[] = FWFM_DIR . 'helpers/fwfmHtml/';
				$paths[] = FWFM_DIR . 'helpers/fwfmhtml/';
			}

			if (!is_null($path)) {
				$path = rtrim($path, '\\/').'/';
				if (!in_array($path, $paths)) {
					$paths[] = $path;
				}
			} else return $paths;
		}
		static function stylesheet($link, $options=null) {
			$version = false;
			if (!empty($options['version'])) {
				$version = preg_replace('#[^0-9.]+#', '', $options['version']);
			}
			$content_folder = fwfmFactory::getContentFolder();
			wp_enqueue_style(md5($link), str_replace(
				array(
					'administrator/components/com_fwfoodmenu',
					'components/com_fwfoodmenu',
					'plugins/fwfoodmenu/',
					'plugins/fwfoodmenutmpl/',
					'modules/'
				),
				array(
					'/'.$content_folder.'/plugins/fw-food-menu/admin',
					'/'.$content_folder.'/plugins/fw-food-menu/site',
					'/'.$content_folder.'/plugins/fw-food-menu/plugins/fwfoodmenu/',
					'/'.$content_folder.'/plugins/fw-food-menu/plugins/fwfoodmenutmpl/',
					'/'.$content_folder.'/plugins/fw-food-menu/modules/'
				),
				$link
			), array(), $version);
		}
		static function script($link) {
			$content_folder = fwfmFactory::getContentFolder();
			wp_enqueue_script(md5($link), str_replace(
				array(
					'administrator/components/com_fwfoodmenu',
					'components/com_fwfoodmenu',
					'plugins/fwfoodmenu/',
					'plugins/fwfoodmenutmpl/',
					'modules/'
				),
				array(
					'/'.$content_folder.'/plugins/fw-food-menu/admin',
					'/'.$content_folder.'/plugins/fw-food-menu/site',
					'/'.$content_folder.'/plugins/fw-food-menu/plugins/fwfoodmenu/',
					'/'.$content_folder.'/plugins/fw-food-menu/plugins/fwfoodmenutmpl/',
					'/'.$content_folder.'/plugins/fw-food-menu/modules/'
				),
				$link
			), array('jquery'));
		}
		static function date($date, $format) {
			return $date;
		}
		static function framework($extras = false) {
			wp_enqueue_script('mootools', FWFM_URL.'media/system/js/mootools-core.js');
			if ($extras) wp_enqueue_script('mootools-more', FWFM_URL.'media/system/js/mootools-more.js');
		}
		static function calendar($value, $name, $id, $format = '%Y-%m-%d', $attribs = null) {
			fwfmHtml::framework();
			$readonly = isset($attribs['readonly']) && $attribs['readonly'] == 'readonly';
			$disabled = isset($attribs['disabled']) && $attribs['disabled'] == 'disabled';
	
			if (is_array($attribs)) $attribs = fwfmArrayHelper::toString($attribs);
			if (is_array($attribs)) $attribs = fwfmArrayHelper::toString($attribs);
			if (!$readonly && !$disabled) {
				wp_enqueue_script('jcalendar', FWFM_URL.'media/system/js/calendar.js');
				wp_enqueue_script('jcalendar-setup', FWFM_URL.'media/system/js/calendar-setup.js');
				wp_enqueue_style('jcalendar', FWFM_URL.'media/system/css/calendar-jos.css');
?>
<script type="text/javascript">
window.addEvent('domready', function() {
	Calendar.setup({
		// Id of the input field
		inputField: "<?php echo $id; ?>",
		// Format of the input field
		ifFormat: "<?php echo $format; ?>",
		// Trigger for the calendar (button ID)
		button: "<?php echo $id; ?>_img",
		// Alignment (defaults to "Bl")
		align: "Tl",
		singleClick: true,
		firstDay: 1
	});
});
</script>
<?php
				return '<div class="input-append"><input type="text" class="hasTooltip" title="' . (0 !== (int) $value ? static::_('date', $value, null, null) : '')
					. '" name="' . $name . '" id="' . $id . '" value="' . htmlspecialchars($value, ENT_COMPAT, 'UTF-8') . '" ' . $attribs . ' />'
					. '<button type="button" class="btn" id="' . $id . '_img"><i class="icon-calendar"></i></button></div>';
			} else {
				return '<input type="text" class="hasTooltip" title="' . (0 !== (int) $value ? static::_('date', $value, null, null) : '')
					. '" value="' . (0 !== (int) $value ? static::_('date', $value, 'Y-m-d H:i:s', null) : '') . '" ' . $attribs
					. ' /><input type="hidden" name="' . $name . '" id="' . $id . '" value="' . htmlspecialchars($value, ENT_COMPAT, 'UTF-8') . '" />';
			}
		}
		public static function formvalidation() {
			fwfmHtml::framework();
			wp_enqueue_script('formvalidation-puny', FWFM_URL.'media/system/js/punycode.js');
			wp_enqueue_script('formvalidation', FWFM_URL.'media/system/js/validate.js');
		}
		public static function modal($selector = 'a.modal', $params = array()) {
			fwfmHtml::framework();
			wp_enqueue_script('modal', FWFM_URL.'media/system/js/modal.js');
			wp_enqueue_stylesheet('modal', FWFM_URL.'media/system/css/modal.css');
	
			// Setup options object
			$opt['ajaxOptions']   = (isset($params['ajaxOptions']) && (is_array($params['ajaxOptions']))) ? $params['ajaxOptions'] : null;
			$opt['handler']       = (isset($params['handler'])) ? $params['handler'] : null;
			$opt['parseSecure']   = (isset($params['parseSecure'])) ? (bool) $params['parseSecure'] : null;
			$opt['closable']      = (isset($params['closable'])) ? (bool) $params['closable'] : null;
			$opt['closeBtn']      = (isset($params['closeBtn'])) ? (bool) $params['closeBtn'] : null;
			$opt['iframePreload'] = (isset($params['iframePreload'])) ? (bool) $params['iframePreload'] : null;
			$opt['iframeOptions'] = (isset($params['iframeOptions']) && (is_array($params['iframeOptions']))) ? $params['iframeOptions'] : null;
			$opt['size']          = (isset($params['size']) && (is_array($params['size']))) ? $params['size'] : null;
			$opt['shadow']        = (isset($params['shadow'])) ? $params['shadow'] : null;
			$opt['overlay']       = (isset($params['overlay'])) ? $params['overlay'] : null;
			$opt['onOpen']        = (isset($params['onOpen'])) ? $params['onOpen'] : null;
			$opt['onClose']       = (isset($params['onClose'])) ? $params['onClose'] : null;
			$opt['onUpdate']      = (isset($params['onUpdate'])) ? $params['onUpdate'] : null;
			$opt['onResize']      = (isset($params['onResize'])) ? $params['onResize'] : null;
			$opt['onMove']        = (isset($params['onMove'])) ? $params['onMove'] : null;
			$opt['onShow']        = (isset($params['onShow'])) ? $params['onShow'] : null;
			$opt['onHide']        = (isset($params['onHide'])) ? $params['onHide'] : null;
	
			// Include jQuery
			fwfmHtml::_('jquery.framework');
	
			if (isset($params['fullScreen']) && (bool) $params['fullScreen'])
			{
				$opt['size']      = array('x' => '\\jQuery(window).width() - 80', 'y' => '\\jQuery(window).height() - 80');
			}
	
			$options = fwfmHtml::getJSObject($opt);
	
			// Attach modal behavior to document
			$document->addScriptDeclaration("
document.addEventListener('DOMContentLoaded', function() {
	(function($) {
		SqueezeBox.initialize(" . $options . ");
		SqueezeBox.assign($('" . $selector . "').get(), {
			parse: 'rel'
		});
	})(jQuery);
});"
			);
	
			// Set static array
			static::$loaded[__METHOD__][$sig] = true;
			return;
		}
	}
}

if (!class_exists('fwfmUser')) {
	class fwfmUser extends fwfmObject {
		var $id = 0,
		$username = null,
		$name = null,
		$email = null;
	
		function bind() {
		}
		function check() {
		}
		function store() {
		}
		function delete() {
		}
	}
}

if (!class_exists('fwfmFactory')) {
	class fwfmFactory {
		static function getContentFolder() {
			static $folder = null;
			if (is_null($folder)) {
				$buff = explode('/', str_replace('\\', '/', WP_CONTENT_DIR));
				$folder = $buff[count($buff)-1];
			}
			return $folder;
		}
		static function getConfig() {
			$config = new \fwfmRegistry;
			return $config;
		}
		static function getCache() {
			static $cache;
			if (!$cache) $cache = new \fwfmCache;
			return $cache;
		}
		static function getDBO() {
			static $db;
			if (!$db) $db = new \fwfmDB;
			return $db;
		}
		static function getUser() {
			$data = wp_get_current_user();
	
			$user = new \fwfmUser;
			$user->id = $data->id;
			$user->username = $data->name;
			$user->name = $data->first_name .' '.$data->last_name;
			$user->email = $data->user_email;
	
			return $user;
		}
		static function getApplication() {
			static $app;
			if (!$app) $app = new \fwfmApplication;
			return $app;
		}
		static function getDocument() {
			static $doc;
			if (!$doc) $doc = new \fwfmDocument;
			return $doc;
		}
		static function getLanguage() {
			static $lang;
			if (!$lang) $lang = new \fwfmLanguage;
			return $lang;
		}
		static function getEditor() {
			static $editor;
			if (!$editor) $editor = new \fwfmEditor;
			return $editor;
		}
		static function getSession() {
			static $session;
			if (!$session) $session = new \fwfmSession;
			return $session;
		}
		static function getMailer() {
			static $mailer;
			if (!$mailer) $mailer = new \fwfmMailer;
			return $mailer;
		}
		static function getDate($time='now') {
			static $date = array();
			if (!isset($date[$time])) $date[$time] = new \fwfmDate($time);
			return $date[$time];
		}
	}
}

if (!class_exists('fwfmDate')) {
	class fwfmDate {
		var $time;
		function __construct($time) {
			$this->time = is_numeric($time)?$time:strtotime($time);
		}
		function toSql() {
			return date('Y-m-d H:i:s', $this->time);
		}
	}
}

if (!class_exists('fwfmMailer')) {
	class fwfmMailer {
		function sendMail($fromname, $emailfrom, $to, $subject, $message, $mode=false, $cc=null, $bcc=null, $attachments=null, $reply_to=null, $reply_to_name=null) {
			$headers = array('From: '.$fromname.' <'.$emailfrom.'>');
			if ($cc) {
				$headers[] = 'Cc:'.$cc;
			}
			if ($bcc) {
				$headers[] = 'Bcc:'.$bcc;
			}
			if ($reply_to and $reply_to_name) {
				$headers[] = 'Reply-To: '.$reply_to_name.' <'.$reply_to.'>';
			} elseif ($reply_to) {
				$headers[] = 'Reply-To:'.$reply_to;
			}
			if ($mode) {
				$headers[] = 'Content-Type: text/html; charset=UTF-8';
			}
			return wp_mail($to, $subject, $message, $headers, $attachments);
		}
	}
}

if (!class_exists('fwfmSession')) {
	class fwfmSession {
		function getId() {
			return session_id();
		}
	}
}

if (!class_exists('fwfmModuleHelper')) {
	class fwfmModuleHelper {
		function getLayoutPath($module_name, $layout='default') {
			return FWFM_DIR.'modules/'.$module_name.'/tmpl/'.$layout.'.php';
		}
	}
}

if (!class_exists('fwfmEditor')) {
	class fwfmEditor {
		var $type;
		var $instances = array();
		static function getInstance($type = null) {
			if (!$type) {
				$type = 'common';
			}
			if (!isset($instances[$type])) {
				if ($type == 'codemirror') {
					wp_enqueue_script( 'code-editor' );
					wp_enqueue_style( 'code-editor' );
	
					wp_enqueue_script('csslint');
				}
				$instances[$type] = new \fwfmEditor($type);
			}
			return $instances[$type];
		}
		function __construct($type = null) {
			$this->type = $type;
		}
		function display($name, $html, $width=600, $height=400, $cols=60, $rows=5, $butts=false, $id = null, $asset = null, $author = null, $params = array()) {
			if ($this->type == 'codemirror') {
				$rnd = rand();
				return '<textarea name="'.$name.'" class="form-control" id="fwfm-mc-'.$rnd.'">'.$html.'</textarea>
<script>
jQuery(function($) {
	var editorSettings = {codemirror:{lineNumbers:true,mode:\'css\''.(empty($params['readonly'])?'':',readOnly:true').'}};
	wp.codeEditor.initialize( $(\'#fwfm-mc-'.$rnd.'\'), editorSettings );
})
</script>';
			} else {
				return wp_editor($html, $name);
			}
		}
	}
}

if (!class_exists('fwfmLanguage')) {
	class fwfmLanguage {
		var $lang = '';
		var $langs = array();
		function __construct($lang='') {
			if ($lang == '') {
				$curr_lang = get_locale();
				if ($curr_lang  == 'ar') {
					$lang = 'ar-AA';
				} elseif ($curr_lang == 'el') {
					$lang = 'el-GR';
				} else {
					$lang = str_replace('_', '-', $curr_lang);
				}
				$path = FWFM_CONTROLLER_DIR.'language/'.$lang.'/'.$lang.'.wp.ini';
				if (!file_exists($path)) {
					$lang = 'en-GB';
				}
			}
			$this->lang = $lang;
			$this->langs = array();
			$this->load();
			$app = fwfmFactory::getApplication();
			if ($app->isAdmin()) {
				$this->load('com_fwfoodmenu.sys');
				$this->load('com_fwfoodmenu');
			} else {
				$this->load('com_fwfoodmenu');
			}
			$path = FWFM_CONTROLLER_DIR.'language/overrides/'.$lang.'.wp.override.ini';
			if (file_exists($path)) {
				$data = explode("\n", file_get_contents($path));
				foreach ($data as $row) if ($row) {
					$cols = explode('=', $row, 2);
					if (empty($cols[0]) or (!empty($cols[0]) and $cols[0][0] == '#') or empty($cols[1])) continue;
					$cols[1] = preg_replace('#^"|"$#', '', trim($cols[1]));
					$cols[1] = str_replace(array('\"', '"_QQ_"'), array('"', '"'), $cols[1]);
					$this->langs[strtoupper($cols[0])] = $cols[1];
				}
			}
		}
		function getInstance($lang) {
			static $langs = array();
			if (!isset($langs[$lang])) {
				$langs[$lang] = new \fwfmLanguage($lang);
			}
			return $langs[$lang];
		}
		function transliterate($str) {
			return url_slug($str, $options = array('transliterate'=>true, 'lowercase'=>false, 'delimiter'=>' '));
		}
		function load($name='', $path=FWFM_PATH_BASE) {
			$filename = '';
			if (strpos($name, 'plg_') !== false) {
				$filename = rtrim(FWFM_PATH_BASE, '\\/').'/admin/language/'.$this->lang.'/'.$this->lang.'.'.$name.'.ini';
			} elseif (strpos($name, 'mod_') !== false) {
				$filename = rtrim(FWFM_PATH_BASE, '\\/').'/site/language/'.$this->lang.'/'.$this->lang.'.'.$name.'.ini';
			} else {
				if (fwfmFactory::getApplication()->isAdmin()) {
					$filename = rtrim(FWFM_PATH_BASE, '\\/').'/admin/language/'.$this->lang.'/'.$this->lang.($name?('.'.$name):'').'.ini';
				} else {
					$filename = rtrim(FWFM_PATH_BASE, '\\/').'/site/language/'.$this->lang.'/'.$this->lang.($name?('.'.$name):'').'.ini';
				}
			}
			if ($filename and file_exists($filename)) {
				$data = explode("\n", file_get_contents($filename));
				foreach ($data as $row) if ($row) {
					$cols = explode('=', $row, 2);
					if (empty($cols[0]) or (!empty($cols[0]) and $cols[0][0] == '#') or empty($cols[1])) continue;
					$cols[1] = preg_replace('#^"|"$#', '', trim($cols[1]));
					$cols[1] = str_replace(array('\"', '"_QQ_"', '\\n'), array('"', '"', "\n"), $cols[1]);
					$this->langs[strtoupper($cols[0])] = $cols[1];
				}
			}
	
			$filename = '';
			if (strpos($name, 'plg_') !== false) {
				$filename = rtrim(FWFM_PATH_BASE, '\\/').'/admin/language/'.$this->lang.'/'.$this->lang.'.wp.'.$name.'.ini';
			} elseif (strpos($name, 'mod_') !== false) {
				$filename = rtrim(FWFM_PATH_BASE, '\\/').'/site/language/'.$this->lang.'/'.$this->lang.'.wp.'.$name.'.ini';
			} else {
				if (fwfmFactory::getApplication()->isAdmin()) {
					$filename = rtrim(FWFM_PATH_BASE, '\\/').'/admin/language/'.$this->lang.'/'.$this->lang.'.wp'.($name?('.'.$name):'').'.ini';
				} else {
					$filename = rtrim(FWFM_PATH_BASE, '\\/').'/site/language/'.$this->lang.'/'.$this->lang.'.wp'.($name?('.'.$name):'').'.ini';
				}
			}
			if ($filename and file_exists($filename)) {
				$data = explode("\n", file_get_contents($filename));
				foreach ($data as $row) if ($row) {
					$cols = explode('=', $row, 2);
					if (empty($cols[0]) or (!empty($cols[0]) and $cols[0][0] == '#') or empty($cols[1])) continue;
					$cols[1] = preg_replace('#^"|"$#', '', trim($cols[1]));
					$cols[1] = str_replace(array('\"', '"_QQ_"', '\\n'), array('"', '"', "\n"), $cols[1]);
					$this->langs[strtoupper($cols[0])] = $cols[1];
				}
			}
		}
		function getTag() {
			$curr_lang = get_locale();
			$locales = array(
				'ar' => array('tag' => 'ar-AA', 'name' => 'Arabic') ,
				'de_DE' => array('tag' => 'de-DE', 'name' => 'German') ,
				'el' => array('tag' => 'el-GR', 'name' => 'Greek') ,
				'es_ES' => array('tag' => 'es-ES', 'name' => 'Spanish') ,
				'en_GB' => array('tag' => 'en-GB', 'name' => 'English (GB)') ,
				'en_US' => array('tag' => 'en-US', 'name' => 'English (US)') ,
				'fr_FR' => array('tag' => 'fr-FR', 'name' => 'French') ,
				'it_IT' => array('tag' => 'it-IT', 'name' => 'Italian') ,
				'pt_PT' => array('tag' => 'pt-PT', 'name' => 'pt_PT') ,
				'ru_RU' => array('tag' => 'ru-RU', 'name' => 'Russian') ,
				'tr_TR' => array('tag' => 'tr-TR', 'name' => 'Turkish') ,
				'ru_UA' => array('tag' => 'uk-UA', 'name' => 'Ukrainian') ,
				'zh_CN' => array('tag' => 'zh-CN', 'name' => 'Chinese') ,
			);
			if (!empty($locales[$curr_lang])) {
				return $locales[$curr_lang]['tag'];
			} else {
				return str_replace('_', '-', $curr_lang);
			}
		}
		function isRTL() {
			return is_rtl();
		}
		static function getKnownLanguages() {
			$locales = array(
				'ar' => array('tag' => 'ar-AA', 'name' => 'Arabic') ,
				'de_DE' => array('tag' => 'de-DE', 'name' => 'German') ,
				'el' => array('tag' => 'el-GR', 'name' => 'Greek') ,
				'es_ES' => array('tag' => 'es-ES', 'name' => 'Spanish') ,
				'en_GB' => array('tag' => 'en-GB', 'name' => 'English (GB)') ,
				'en_US' => array('tag' => 'en-US', 'name' => 'English (US)') ,
				'fr_FR' => array('tag' => 'fr-FR', 'name' => 'French') ,
				'it_IT' => array('tag' => 'it-IT', 'name' => 'Italian') ,
				'pt_PT' => array('tag' => 'pt-PT', 'name' => 'pt_PT') ,
				'ru_RU' => array('tag' => 'ru-RU', 'name' => 'Russian') ,
				'tr_TR' => array('tag' => 'tr-TR', 'name' => 'Turkish') ,
				'ru_UA' => array('tag' => 'uk-UA', 'name' => 'Ukrainian') ,
				'zh_CN' => array('tag' => 'zh-CN', 'name' => 'Chinese') ,
			);
	
			$langs = array(
				array(
					'tag' => 'en-GB',
					'name' => 'English'
				)
			);
			$content_folder = fwfmFactory::getContentFolder();
			$path = ABSPATH.'/'.$content_folder.'/languages';
			$files = fwfmFolder::files($path, '^\w{2}_{0,1}\w{0,2}\.mo');
			if ($files) {
				sort($files);
				foreach ($files as $filename) {
					$name = preg_replace('#\.mo#', '', $filename);
					$langs[] = array(
						'tag' => empty($locales[$name])?str_replace('_', '-', $name):$locales[$name]['tag'],
						'name' => empty($locales[$name])?$name:$locales[$name]['name']
					);
				}
			} else {
				$curr_lang = str_replace('_', '-', get_locale());
				if ($curr_lang != 'en-GB') {
					if (!empty($locales[$curr_lang])) {
						$langs[] = $locales[$curr_lang];
					} elseif ($curr_lang != 'en-US') {
						$langs[] = array(
							'tag' => $curr_lang,
							'name' => $curr_lang
						);
					}
				}
			}
			return $langs;
		}
	}
}

if (!class_exists('fwfmLanguageHelper')) {
	class fwfmLanguageHelper {
		static function getKnownLanguages() {
			$lang = fwfmFactory::getLanguage();
			return $lang->getKnownLanguages();
		}
	}
}

if (!class_exists('fwfmURI')) {
	class fwfmURI {
		static function getInstance() {
			static $instance = null;
			if (is_null($instance)) {
				$instance = new \fwfmURI();
			}
			return $instance;
		}
	
		static function root($is_relative=true) {
			$content_folder = fwfmFactory::getContentFolder();
			return site_url().'/'.$content_folder.'/plugins/fw-food-menu'.($is_relative?'':'/');
		}
		static function base($is_relative=true) {
			$content_folder = fwfmFactory::getContentFolder();
			return site_url().'/'.$content_folder.'/plugins/fw-food-menu'.($is_relative?'':'/');
		}
		static function toString() {
			return (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
		}
	}
}

if (!class_exists('fwfmTable')) {
	class fwfmTable extends fwfmObject {
		var $_tb_name = null,
			$_tb_key = null,
			$_db = null;
		static function addIncludePath() {}
		static function getInstance($name, $prefix='Table') {
			static $tables = array();
			if (!isset($tables[$name.'-'.$prefix])) {
				$name = strtolower($name);
				$path = FWFM_DIR.'admin/tables/'.$name.'.php';
				if (file_exists($path)) {
					require_once($path);
					$class_name = $prefix.$name;
					if (class_exists($class_name)) {
						$db = fwfmFactory::getDBO();
						$class_name = '\\'.$class_name;
						$tables[$name.'-'.$prefix] = new $class_name($db);
					}
				}
			}
			if (isset($tables[$name.'-'.$prefix])) return $tables[$name.'-'.$prefix];
		}
		function __construct($name, $key, $db) {
			$this->_tb_name = $name;
			$this->_tb_key = $key;
			$this->_db = fwfmFactory::getDBO();
		}
		function bind($var) {
			$keys = array_keys($this->getProperties());
			foreach ($var as $key=>$val) {
				if (in_array($key, $keys)) {
					$this->$key = JFilterInput::sanitazeHtml($val);
				}
			}
			return true;
		}
		function load($id = null, $reset=false) {
			if ($id) {
				$this->_db->setQuery('SELECT * FROM `'.$this->_tb_name.'` WHERE `'.$this->_tb_key.'` = '.(int)$id);
				if ($obj = $this->_db->loadObject()) {
					foreach ($obj as $key=>$val) {
						$this->$key = $val;
					}
					return true;
				}
			}
		}
		function check() {
			return true;
		}
		function store($updn = false) {
			$query = '';
			foreach ($this as $key=>$val) {
				if ($key != $this->_tb_key and $key[0] != '_' and (!is_null($val) or $updn)) {
					$query .= ($query?',':'').'`'.$key.'`='.(is_numeric($val)?$val:$this->_db->quote($val));
				}
			}
			if ($query) {
				if ($this->{$this->_tb_key}) $this->_db->setQuery('UPDATE '.$this->_tb_name.' SET '.$query.' WHERE `'.$this->_tb_key.'` = '.$this->{$this->_tb_key});
				else $this->_db->setQuery('INSERT INTO '.$this->_tb_name.' SET '.$query);
				if ($this->_db->query()) {
					if (!$this->{$this->_tb_key}) {
						$this->{$this->_tb_key} = $this->_db->insertid();
					}
					return true;
				} else $this->setError($this->_db->getError());
			}
		}
		function hit() {
			$keys = array_keys($this->getProperties());
			if (in_array('hits', $keys) and !empty($this->{$this->_tb_key})) {
				$this->_db->setQuery('UPDATE `'.$this->_tb_name.'` SET hits = hits + 1 WHERE `'.$this->_tb_key.'` = '.(int)$this->{$this->_tb_key});
				if ($this->_db->execute()) {
					return true;
				} else {
					$this->setError($this->_db->getError());
				}
			}
		}
		function move($dir, $where='') {
			$keys = array_keys($this->getProperties());
			if (in_array('ordering', $keys) and !empty($this->{$this->_tb_key})) {
				if ($dir > 0) {
					$this->_db->setQuery('SELECT `'.$this->_tb_key.'`, `ordering` FROM `'.$this->_tb_name.'` WHERE `ordering` > '.(int)$this->ordering.' LIMIT 1');
				} else {
					$this->_db->setQuery('SELECT `'.$this->_tb_key.'`, `ordering` FROM `'.$this->_tb_name.'` WHERE `ordering` < '.(int)$this->ordering.' LIMIT 1');
				}
				if ($obj = $this->_db->loadObject()) {
					$this->_db->setQuery('UPDATE `'.$this->_tb_name.'` SET `ordering` = '.(int)$this->ordering.' WHERE `'.$this->_tb_key.'` = '.(int)$obj->{$this->_tb_key});
					$this->_db->execute();
					$this->_db->setQuery('UPDATE `'.$this->_tb_name.'` SET `ordering` = '.(int)$obj->ordering.' WHERE `'.$this->_tb_key.'` = '.(int)$this->{$this->_tb_key});
					$this->_db->execute();
					return true;
				}
			}
		}
		function reorder($where = '') {
			$keys = array_keys($this->getProperties());
			if (in_array('ordering', $keys)) {
				$this->_db->setQuery('SELECT `'.$this->_tb_key.'`, `ordering` FROM `'.$this->_tb_name.'`'.($where?(' WHERE '.$where):'').' ORDER BY `ordering`');
				if ($list = $this->_db->loadObjectList()) {
					foreach ($list as $i=>$row) {
						$num = $i + 1;
						if ($row->ordering != $num) {
							$this->_db->setQuery('UPDATE `'.$this->_tb_name.'` SET `ordering` = '.$num.' WHERE `'.$this->_tb_key.'` = '.$row->{$this->_tb_key});
							$this->_db->execute();
						}
					}
					return true;
				}
			}
		}
		function publish($ids, $publish) {
			$keys = array_keys($this->getProperties());
			if ($ids and in_array('published', $keys)) {
				if (!is_array($ids)) $ids = (array)$ids;
				fwfmArrayHelper::toInteger($ids, 0);
				$this->_db->setQuery('UPDATE `'.$this->_tb_name.'` SET `published` = '.($publish?'1':'0').' WHERE `'.$this->_tb_key.'` IN ('.implode(',', $ids).')');
				if ($this->_db->execute()) {
					return true;
				} else {
					$this->setError($this->_db->getError());
				}
			}
		}
		function delete($id) {
			if ($id) {
				$this->_db->setQuery('DELETE FROM '.$this->_tb_name.' WHERE `'.$this->_tb_key.'` = '.(int)$id);
				if ($this->_db->execute()) {
					return true;
				} else {
					$this->setError($this->_db->getError());
				}
			}
		}
	}
}

if (!class_exists('fwfmModelLegacy')) {
	class fwfmModelLegacy {
		var $name = null,
			$errors = array();
		static function getInstance($name, $prefix=null) {
			static $models = array();
	
			if (is_null($prefix)) $prefix = 'itemsModel';
			$name = strtolower($name);
	
			if (!isset($models[$name])) {
				$paths = self::addIncludePath();
				foreach ($paths as $path) {
					if (file_exists($path.'/'.$name.'.php')) {
						require_once($path.'/'.$name.'.php');
						$class_name = '\\'.$prefix.$name;
						if (class_exists($class_name)) $models[$name] = new $class_name($name);
					}
				}
			}
			if (!isset($models[$name])) {
				$path = FWFM_DIR.(fwfmFactory::getApplication()->isAdmin()?'admin':'site').'/models/'.$name.'.php';
				if (file_exists($path)) {
					require_once($path);
					$class_name = '\\'.$prefix.$name;
					if (class_exists($class_name)) $models[$name] = new $class_name($name);
				}
			}
			if (isset($models[$name])) return $models[$name];
		}
		function __construct($name) {
			$this->name = $name;
		}
		function getTable($name) {
			return fwfmTable::getInstance($name);
		}
		function setError($error) {
			$this->errors[] = $error;
		}
		function getError() {
			if ($this->errors) return $this->errors[count($this->errors) - 1];
		}
		function getErrors() {
			return $this->errors;
		}
		static function addIncludePath($path=null) {
			static $paths;

			if (!isset($paths)) {
				$paths = array();
			}

			if (!empty($path) && !in_array($path, $paths)) {
				array_unshift($paths, $path);
			}

			return $paths;
		}
	}
}

if (!class_exists('fwfmViewLegacy')) {
	class fwfmViewLegacy {
		var $name = null;
		var $model = null;
		var $displayed = false;
		static function getInstance($name, $prefix=null) {
			static $views = array();
	
			if (is_null($prefix)) $prefix = 'itemsView';
	
			$format = fwfmRequest::getString('format', 'html');
			if (!$format) $format = 'html';
	
			$name = strtolower($name);
			$format = strtolower($format);
	
			if (!isset($views[$name.'-'.$format])) {
				$path = FWFM_DIR.(fwfmFactory::getApplication()->isAdmin()?'admin':'site').'/views/'.$name.'/view.'.$format.'.php';
	
				if (file_exists($path)) {
					require_once($path);
					$class_name = '\\'.$prefix.$name;
					if (class_exists($class_name)) $views[$name.'-'.$format] = new $class_name($name);
				}
			}
			if (isset($views[$name.'-'.$format])) {
				return $views[$name.'-'.$format];
			}
		}
		function __construct($name) {
			$this->name = $name;
			$this->layout = fwfmRequest::getString('layout');
			$this->model = fwfmModelLegacy::getInstance($this->name, 'itemsModel');
		}
		function assign($key, $val) {
			$this->$key = $val;
		}
		function assignRef($key, $val) {
			$this->$key = &$val;
		}
		function setModel($model) {
			return $this->model = $model;
		}
		function setLayout($layout) {
			$this->layout = $layout;
		}
		function getLayout() {
			return $this->layout;
		}
		function getModel() {
			return $this->model;
		}
		function escape($val) {
			return str_replace('"', '&quot;', $val);
		}
		function loadTemplate($tmpl=null) {
			$app = fwfmFactory::getApplication();
			$path = FWFM_DIR.($app->isAdmin()?'admin':'site').'/views/'.$this->name.'/tmpl/'.$this->layout.($tmpl?('_'.$tmpl):'').'.php';
			if ($this->layout != 'default' and !file_exists($path)) {
				$path = FWFM_DIR.($app->isAdmin()?'admin':'site').'/views/'.$this->name.'/tmpl/default.php';
			}
			ob_start();
			if (file_exists($path)) require($path);
			return ob_get_clean();
		}
		function display() {
			if (fwfmFactory::getApplication()->isAdmin() and $this->displayed) return;
			$this->displayed = true;
	
			if (!$this->layout) $this->layout = 'default';
			echo $this->loadTemplate();
		}
	}
}

if (!class_exists('fwfmCache')) {
	class fwfmCache {
		function clean() {}
	}
}

if (!class_exists('fwfmSubMenuHelper')) {
	class fwfmSubMenuHelper {
		function addEntry() {}
	}
}

if (!class_exists('fwfmText')) {
	class fwfmText {
		static function _($text) {
			$lang = fwfmFactory::getLanguage();
			return isset($lang->langs[strtoupper($text)])?$lang->langs[strtoupper($text)]:$text;
		}
		static function sprintf($text) {
			$args = func_get_args();
			if (count($args)) {
				$args[0] = fwfmText::_($args[0]);
				return call_user_func_array('sprintf', $args);
			}
		}
	}
}

if (!class_exists('fwfmRoute')) {
	class fwfmRoute {
		static function _($url) {
			global $wp;
			static $urls = array();
	
			if ($url[0] == '&') {
				$url = '?option=com_fwfoodmenu'.$url;
			}
			$url = str_replace('index.php', '', $url);
			if (preg_match('#view=menu&id=(\d+)#', $url, $match)) {
				if (!isset($urls[$match[1]])) {
					$category_found = false;
					/* check current post */
					$post = get_post();
					if (!empty($post->post_content)) {
						if (preg_match_all('#\[fwfm_menu[^\]]+category_id="([^"]+)"#i', $post->post_content, $matches, PREG_SET_ORDER)) {
							foreach ($matches as $row) {
								$buff = explode(',', $row[1]);
								if (in_array($match[1], $buff)) {
									$urls[$match[1]] = home_url($wp->request);
									$category_found = true;
									break;
								}
							}
						}
					}
					if (!$category_found) {
						$db = fwfmFactory::getDBO();
						$db->setQuery('SELECT ID, post_content FROM `#__posts` WHERE post_type IN (\'post\', \'page\') AND `post_status` = \'publish\' AND post_content LIKE \'%category_id%'.(int)$match[1].'%\'');
						if ($list = $db->loadObjectList()) {
							foreach ($list as $post_row) {
								if (preg_match_all('#\[fwfm_menu[^\]]+category_id="([^"]+)"#i', $post_row->post_content, $matches, PREG_SET_ORDER)) {
									foreach ($matches as $row) {
										$buff = explode(',', $row[1]);
										if (in_array($match[1], $buff)) {
											$urls[$match[1]] = get_permalink($post_row->ID);
											$category_found = true;
											break;
										}
									}
								}
							}
						}
					}
					if (!$category_found) {
						$urls[$match[1]] = false;
					}
				}
				if ($urls[$match[1]]) {
					$url = $urls[$match[1]];
				}
			}
			return fwfmFactory::getApplication()->isAdmin()?admin_url($url):$url;
		}
		static function fwwpFixLinks($buff) {
			if (fwfmFactory::getApplication()->isAdmin()) {
				$content_folder = fwfmFactory::getContentFolder();
				return str_replace(array(
					'index.php?option=com_fwfoodmenu&amp;view=',
					'index.php?option=com_fwfoodmenu&view=',
					'index.php?option=com_ajax?',
					'index.php?option=com_ajax',
					'?option=com_ajax',
					'administrator/components/com_fwfoodmenu/',
					'components/com_fwfoodmenu/',
					'dataType: \'json\',',
					$content_folder.'/plugins/fw-food-menu/admin-ajax.php'
				), array(
					'admin.php?page=fwfm-',
					'admin.php?page=fwfm-',
					'admin-ajax.php?action=fwajax&',
					'admin-ajax.php?action=fwajax',
					'admin-ajax.php?action=fwajax',
					'admin/',
					'site/',
					'dataType: \'json\', url: \'admin-ajax.php?action=fwajax\',',
					'wp-admin/admin-ajax.php'
				), $buff);
			} else {
				return str_replace(array(
					'index.php?option=com_ajax?',
					'index.php?option=com_ajax',
					'?option=com_ajax',
					'administrator/components/com_fwfoodmenu/',
					'components/com_fwfoodmenu/',
					'dataType: \'html\',',
					'src="images/',
					'url(images/'
				), array(
					site_url().'/wp-admin/admin-ajax.php?action=fwajax&',
					site_url().'/wp-admin/admin-ajax.php?action=fwajax',
					site_url().'/wp-admin/admin-ajax.php?action=fwajax',
					fwfmURI::root(true).'/admin/',
					fwfmURI::root(true).'/site/',
					'dataType: \'html\', url: \''.site_url().'/wp-admin/admin-ajax.php?action=fwajax\',',
					'src="'.fwfmURI::root(true).'/images/',
					'url('.fwfmURI::root(true).'/images/'
				), $buff);
			}
		}
	}
}

if (!class_exists('fwfmArrayHelper')) {
	class fwfmArrayHelper {
		protected static $sortCase;
		protected static $sortDirection;
		protected static $sortKey;
		protected static $sortLocale;
		static function toInteger(&$data, $default=0) {
			foreach ($data as $i=>$val) {
				$data[$i] = is_numeric($val)?(int)$val:$default;
			}
		}
		static function toArray($data) {
			$data = (array)$data;
			foreach ($data as $key=>$val) {
				if (is_object($val)) {
					$data[$key] = fwfmArrayHelper::toArray($val);
				}
			}
			return $data;
		}
		static function toString($data) {
			return implode(' ', (array)$data);
		}
		static function getValue($data, $name, $default='') {
			if (isset($data[$name])) return $data[$name];
			else return $default;
		}
		public static function sortObjects(&$a, $k, $direction = 1, $caseSensitive = true, $locale = false) {
			if (!is_array($locale) || !is_array($locale[0]))
			{
				$locale = array($locale);
			}
	
			self::$sortCase = (array) $caseSensitive;
			self::$sortDirection = (array) $direction;
			self::$sortKey = (array) $k;
			self::$sortLocale = $locale;
	
			usort($a, array(__CLASS__, '_sortObjects'));
	
			self::$sortCase = null;
			self::$sortDirection = null;
			self::$sortKey = null;
			self::$sortLocale = null;
	
			return $a;
		}
		protected static function _sortObjects(&$a, &$b)
		{
			$key = self::$sortKey;
	
			for ($i = 0, $count = count($key); $i < $count; $i++)
			{
				if (isset(self::$sortDirection[$i]))
				{
					$direction = self::$sortDirection[$i];
				}
	
				if (isset(self::$sortCase[$i]))
				{
					$caseSensitive = self::$sortCase[$i];
				}
	
				if (isset(self::$sortLocale[$i]))
				{
					$locale = self::$sortLocale[$i];
				}
	
				$va = $a->{$key[$i]};
				$vb = $b->{$key[$i]};
	
				if ((is_bool($va) || is_numeric($va)) && (is_bool($vb) || is_numeric($vb)))
				{
					$cmp = $va - $vb;
				}
				elseif ($caseSensitive)
				{
					$cmp = fwfmStringHelper::strcmp($va, $vb, $locale);
				}
				else
				{
					$cmp = fwfmStringHelper::strcasecmp($va, $vb, $locale);
				}
	
				if ($cmp > 0)
				{
					return $direction;
				}
	
				if ($cmp < 0)
				{
					return -$direction;
				}
			}
	
			return 0;
		}
	}
}

if (!class_exists('fwfmStringHelper')) {
	class fwfmStringHelper {
		public static function strcasecmp($str1, $str2, $locale = false)
		{
			if ($locale)
			{
				// Get current locale
				$locale0 = setlocale(LC_COLLATE, 0);
	
				if (!$locale = setlocale(LC_COLLATE, $locale))
				{
					$locale = $locale0;
				}
	
				// See if we have successfully set locale to UTF-8
				if (!stristr($locale, 'UTF-8') && stristr($locale, '_') && preg_match('~\.(\d+)$~', $locale, $m))
				{
					$encoding = 'CP' . $m[1];
				}
				elseif (stristr($locale, 'UTF-8') || stristr($locale, 'utf8'))
				{
					$encoding = 'UTF-8';
				}
				else
				{
					$encoding = 'nonrecodable';
				}
	
				// If we successfully set encoding it to utf-8 or encoding is sth weird don't recode
				if ($encoding == 'UTF-8' || $encoding == 'nonrecodable')
				{
					return strcoll(utf8_strtolower($str1), utf8_strtolower($str2));
				}
	
				return strcoll(
					static::transcode(utf8_strtolower($str1), 'UTF-8', $encoding),
					static::transcode(utf8_strtolower($str2), 'UTF-8', $encoding)
				);
			}
	
			return utf8_strcasecmp($str1, $str2);
		}
		public static function strcmp($str1, $str2, $locale = false)
		{
			if ($locale)
			{
				// Get current locale
				$locale0 = setlocale(LC_COLLATE, 0);
	
				if (!$locale = setlocale(LC_COLLATE, $locale))
				{
					$locale = $locale0;
				}
	
				// See if we have successfully set locale to UTF-8
				if (!stristr($locale, 'UTF-8') && stristr($locale, '_') && preg_match('~\.(\d+)$~', $locale, $m))
				{
					$encoding = 'CP' . $m[1];
				}
				elseif (stristr($locale, 'UTF-8') || stristr($locale, 'utf8'))
				{
					$encoding = 'UTF-8';
				}
				else
				{
					$encoding = 'nonrecodable';
				}
	
				// If we successfully set encoding it to utf-8 or encoding is sth weird don't recode
				if ($encoding == 'UTF-8' || $encoding == 'nonrecodable')
				{
					return strcoll($str1, $str2);
				}
	
				return strcoll(static::transcode($str1, 'UTF-8', $encoding), static::transcode($str2, 'UTF-8', $encoding));
			}
	
			return strcmp($str1, $str2);
		}
		public static function transcode($source, $fromEncoding, $toEncoding)
		{
			if (\is_string($source))
			{
				switch (ICONV_IMPL)
				{
					case 'glibc':
						return @iconv($fromEncoding, $toEncoding . '//TRANSLIT,IGNORE', $source);
	
					case 'libiconv':
					default:
						return iconv($fromEncoding, $toEncoding . '//IGNORE//TRANSLIT', $source);
				}
			}
		}
	}
}

if (!class_exists('fwfmMenu')) {
	class fwfmMenu {
		var $item;
		static function getInstance($name) {
			static $menus = array();
			if (!isset($menus[$name])) {
				$menus[$name] = new \fwfmMenu($name);
			}
			return $menus[$name];
		}
		function __construct($name) {
			$this->item = (object)array(
				'id' => 1,
				'title' => '',
				'params' => new \fwfmRegistry
			);
		}
		function getActive() {
			return $this->item;
		}
		function getItem() {
			return $this->item;
		}
		function getItems() {
			return array($this->item);
		}
	}
}

if (!class_exists('fwfmInstallerHelper')) {
	class fwfmInstallerHelper {
		static function unpack($filename) {
			$result = array('dir' => '');
			if (file_exists($filename)) {
				$path = dirname($filename);
				$folder = '';
				do {
					$folder = md5(rand());
				} while (file_exists($path.'/'.$folder));
				fwfmFolder::create($path.'/'.$folder);
	
				WP_Filesystem();
				$buff = unzip_file($filename, $path.'/'.$folder);
				if ($buff) {
					if (is_wp_error($buff)) {
						$result['error'] = $buff->get_error_message().' '.$filename;
					} else {
						$result['dir'] = $path.'/'.$folder;
					}
				}
			}
			return $result;
		}
	}
}

if (!class_exists('fwfmInstaller')) {
	class fwfmInstaller {
		static function getInstance() {
			static $instance = null;
			if (!is_object($instance)) {
				$instance = new \fwfmInstaller;
			}
			return $instance;
		}
		function install($path) {
			$install_xml = '';
			$type = '';
			$files = fwfmFolder::files($path, '\.xml', false, true);
			foreach ($files as $file) {
				$buff = file_get_contents($file);
				if (preg_match('#type="([^"]+)"#', $buff, $match)) {
					$install_xml = $buff;
					$type = $match[1];
					break;
				}
			}
			$this->prepareInstallation($path);
			switch ($type) {
				case 'module' :
				return $this->installModule($path, $install_xml);
				break;
				case 'plugin' :
				return $this->installPlugin($path, $install_xml);
				break;
				case 'component' :
				return $this->installComponent($path, $install_xml);
				break;
			}
		}
		function prepareInstallation($path) {
			$files = fwfmFolder::files($path, '.', true, true);
			foreach ($files as $file) {
				if (is_file($file)) {
					if (strpos($file, '/language/') !== false) {
						$buff = explode('/', $file);
						$lang = $buff[count($buff) - 2];
						$filename = $buff[count($buff) - 1];
						fwfmFile::copy($file, $lang_path.'/'.$lang.'/'.$filename, '', true);
					} else {
						$buff = file_get_contents($file);
						$changed = str_ireplace(array(
							chr(74).'EXEC',
							chr(74).'PATH',
							chr(74).'Plugin',
							chr(74).'Object',
							chr(74).'FParams',
							chr(74).'Db',
							chr(74).'LayoutHelper',
							chr(74).'Document',
							chr(74).'Request',
							chr(74).'Input',
							chr(74).'Application',
							chr(74).'Pagination',
							chr(74).'ToolBarHelper',
							chr(74).'Html',
							chr(74).'User',
							chr(74).'Factory',
							chr(74).'Date',
							chr(74).'Mailer',
							chr(74).'Session',
							chr(74).'ModuleHelper',
							chr(74).'Editor',
							chr(74).'URI',
							chr(74).'Table',
							chr(74).'ViewLegacy',
							chr(74).'ModelLegacy',
							chr(74).'Cache',
							chr(74).'SubMenuHelper',
							chr(74).'Text',
							chr(74).'Route',
							chr(74).'ArrayHelper',
							chr(83).'tringHelper',
							chr(74).'Menu',
							chr(74).'InstallerHelper',
							chr(74).'Installer',
							chr(74).'Registry',
							chr(74).'ComponentHelper',
							chr(74).'Folder',
							chr(74).'File',
							chr(74).'FilterOutput',
							chr(74).'FilterInput',
							chr(74).'ControllerLegacy',
							chr(74).'Dispatcher',
							chr(74).'Http',
							chr(74).'Language'
						), array(
							'FWFM_EXEC',
							'FWFM_PATH',
							'fwfmPlugin',
							'fwfmObject',
							'fwfmParams',
							'fwfmDB',
							'fwfmLayoutHelper',
							'fwfmDocument',
							'fwfmRequest',
							'fwfmInput',
							'fwfmApplication',
							'fwfmPagination',
							'fwfmToolBarHelper',
							'fwfmHtml',
							'fwfmUser',
							'fwfmFactory',
							'fwfmDate',
							'fwfmMailer',
							'fwfmSession',
							'fwfmModuleHelper',
							'fwfmEditor',
							'fwfmURI',
							'fwfmTable',
							'fwfmViewLegacy',
							'fwfmModelLegacy',
							'fwfmCache',
							'fwfmSubMenuHelper',
							'fwfmText',
							'fwfmRoute',
							'fwfmArrayHelper',
							'fwfmfwfmStringHelper',
							'fwfmMenu',
							'fwfmInstallerHelper',
							'fwfmInstaller',
							'fwfmRegistry',
							'fwfmComponentHelper',
							'fwfmFolder',
							'fwfmFile',
							'fwfmFilterOutput',
							'fwfmFilterInput',
							'fwfmControllerLegacy',
							'fwfmDispatcher',
							'fwfmHttp',
							'fwfmLanguage'
						), $buff);
						if ($changed != $buff) {
							file_put_contents($file, $changed);
						}

						$trg_path = str_replace($path, '', $file);
						fwfmFile::copy($file, $module_path.$trg_path);
					}
				}
			}			
		}
		function installComponent($path, $install_xml) {
//			fwfmFolder::delete($path.'/admin/language');
//			fwfmFolder::delete($path.'/site/language');

			fwfmFolder::copy($path.'/admin', FWFM_PATH_SITE.'/admin/', '', true);
			fwfmFolder::copy($path.'/site', FWFM_PATH_SITE.'/site/', '', true);
			fw_plugin_activation();
			return true;
		}
		function installModule($path, $install_xml) {
			$module_name = '';
			$name = '';
			if (preg_match('#filename module="([^"]+)"#', $install_xml, $match)) {
				$module_name = $match[1];
			} else return;
			if (preg_match('#<name>([^<]+)</name>#', $install_xml, $match)) {
				$name = $match[1];
			} else return;
	
			$lang_path = FWFM_PATH_COMPONENT_SITE.'/language';
			$module_path = FWFM_PATH_SITE.'/modules/'.$module_name;
			if (!file_exists($module_path)) {
				fwfmFolder::create($module_path);
			}
			if (file_exists($module_path)) {
				$files = fwfmFolder::files($path, '.', true, true);
				foreach ($files as $file) {
					if (is_file($file)) {
						if (strpos($file, '/language/') !== false) {
							$buff = explode('/', $file);
							$lang = $buff[count($buff) - 2];
							$filename = $buff[count($buff) - 1];
							fwfmFile::copy($file, $lang_path.'/'.$lang.'/'.$filename, '', true);
						} else {
							$trg_path = str_replace($path, '', $file);
							fwfmFile::copy($file, $module_path.$trg_path);
						}
					}
				}
				$db = fwfmFactory::getDBO();
				$query = '`name`='.$db->quote($name).', `type`=\'module\', `element`='.$db->quote($module_name).', `folder`=\'\', `manifest_cache`='.$db->quote($install_xml);
				$db->setQuery('SELECT extension_id FROM `#__extensions` WHERE `element` = '.$db->quote($module_name));
				if ($ext_id = $db->loadResult()) {
					$db->setQuery('UPDATE `#__extensions` SET '.$query.' WHERE extension_id = '.$ext_id);
				} else {
					$db->setQuery('INSERT INTO `#__extensions` SET '.$query);
				}
				$db->execute();
				return true;
			}
		}
		function installPlugin($path, $install_xml) {
			$plugin_name = '';
			$group = '';
			$name = '';
			if (preg_match('#filename plugin="([^"]+)"#', $install_xml, $match)) {
				$plugin_name = $match[1];
			} else return;
			if (preg_match('#group="([^"]+)"#', $install_xml, $match)) {
				$group = $match[1];
			} else return;
			if (preg_match('#<name>([^<]+)</name>#', $install_xml, $match)) {
				$name = $match[1];
			} else return;
	
			$lang_path = FWFM_PATH_COMPONENT_ADMINISTRATOR.'/language';
			$plugin_path = FWFM_PATH_SITE.'/plugins/'.$group.'/'.$plugin_name;
			if (!file_exists($plugin_path)) {
				fwfmFolder::create($plugin_path);
			}
			if (file_exists($plugin_path)) {
				$files = fwfmFolder::files($path, '.', true, true);
				foreach ($files as $file) {
					if (is_file($file)) {
						if (strpos($file, '/language/') !== false) {
							$buff = explode('/', $file);
							$lang = $buff[count($buff) - 2];
							$filename = $buff[count($buff) - 1];
							fwfmFile::copy($file, $lang_path.'/'.$lang.'/'.$filename, '', true);
						} else {
							$trg_path = str_replace($path, '', $file);
							fwfmFile::copy($file, $plugin_path.$trg_path);
						}
					}
				}
				$db = fwfmFactory::getDBO();
				$query = '`name`='.$db->quote($name).', `type`=\'plugin\', `element`='.$db->quote($plugin_name).', `folder`='.$db->quote($group).', `manifest_cache`='.$db->quote($install_xml);
				$db->setQuery('SELECT extension_id FROM `#__extensions` WHERE `folder` = '.$db->quote($group).' AND `element` = '.$db->quote($plugin_name));
				if ($ext_id = $db->loadResult()) {
					$db->setQuery('UPDATE `#__extensions` SET '.$query.' WHERE extension_id = '.$ext_id);
				} else {
					$db->setQuery('INSERT INTO `#__extensions` SET '.$query);
				}
				$db->execute();
				return true;
			}
		}
	}
}

if (!class_exists('fwfmRegistry')) {
	class fwfmRegistry {
		var $buff;
		function __construct($data=null) {
			if (is_string($data)) {
				$this->buff = (object)json_decode($data);
			} elseif (is_array($data)) {
				$this->buff = new \stdclass;
				foreach ($data as $key=>$val) {
					$this->buff->$key = $val;
				}
			} elseif (is_object($data)) {
				if ($data instanceof fwfmRegistry) {
					$this->buff = $data->buff;
				} else {
					$this->buff = $data;
				}
			} else {
				$this->buff = new \stdclass;
			}
		}
		function get($key, $def=null) {
			return isset($this->buff->$key)?$this->buff->$key:$def;
		}
		function set($key, $val) {
			$this->buff->$key = $val;
		}
		function loadString($string) {
			$this->buff = json_decode($string);
		}
		function loadArray($array) {
			foreach ($array as $key=>$val) {
				$this->buff->$key = $val;
			}
		}
		function toString() {
			return json_encode($this->buff);
		}
		function toArray() {
			return fwfmArrayHelper::toArray($this->buff);
		}
		function loadFile($file) {
			if (file_exists($file) and $buff = file_get_contents($file)) {
				$this->buff = json_decode($buff);
			}
		}
	}
}

if (!class_exists('fwfmComponentHelper')) {
	class fwfmComponentHelper {
		static function getParams($name) {
			static $params = array();
			if (!isset($params[$name])) {
				$db = fwfmFactory::getDBO();
				$db->setQuery('SELECT `params` FROM `#__extensions` WHERE `type`=\'component\' AND `element`='.$db->quote($name));
				$params[$name] = new \fwfmRegistry($db->loadResult());
			}
			return $params[$name];
		}
	}
}

if (!class_exists('fwfmFolder')) {
	class fwfmFolder {
		public static function create($path, $mode=0755) {
			if (!file_exists($path)) {
				$result = mkdir($path, $mode, true);
			} else $result = true;
			return $result;
		}
		static function delete($path) {
			if (is_dir($path) and is_writable($path)) {
				$files = array_diff(scandir($path), array('.','..'));
				foreach ($files as $file) {
					(is_dir("$path/$file")) ? fwfmFolder::delete("$path/$file") : fwfmFile::delete("$path/$file");
				}
				return rmdir($path);
			}
		}
		static function copy($src, $dest, $path = '', $force = false, $use_streams = false) {
			@set_time_limit(ini_get('max_execution_time'));
			if ($path) {
				$src  = $path . '/' . $src;
				$dest = $path . '/' . $dest;
			}
	
			$src = rtrim($src, DIRECTORY_SEPARATOR);
			$dest = rtrim($dest, DIRECTORY_SEPARATOR);
	
			if (!file_exists($src)) {
				throw new \RuntimeException('Source folder not found', -1);
			}
	
			if (file_exists($dest) && !$force) {
				throw new \RuntimeException('Destination folder already exists', -1);
			}
	
			// Make sure the destination exists
			if (!self::create($dest)) {
				throw new \RuntimeException('Cannot create destination folder', -1);
			}
	
			if (!($dh = @opendir($src)))
			{
				throw new \RuntimeException('Cannot open source folder', -1);
			}
			// Walk through the directory copying files and recursing into folders.
			while (($file = readdir($dh)) !== false) {
				$sfid = $src . '/' . $file;
				$dfid = $dest . '/' . $file;
	
				switch (filetype($sfid)) {
					case 'dir':
						if ($file != '.' && $file != '..') {
							$ret = self::copy($sfid, $dfid, null, $force, $use_streams);
	
							if ($ret !== true) {
								return $ret;
							}
						}
						break;
					case 'file':
						if (!@copy($sfid, $dfid)) {
							throw new \RuntimeException('Copy file failed', -1);
						}
						break;
				}
			}
			return true;
		}
		static function folders($path, $filter = '.', $recurse = false, $full = false, $exclude = array('.svn', 'CVS', '.DS_Store', '__MACOSX'), $excludefilter = array('^\..*')) {
			if (!is_dir($path)) {
				return array();
			}
			if (count($excludefilter)) {
				$excludefilter_string = '/(' . implode('|', $excludefilter) . ')/';
			} else {
				$excludefilter_string = '';
	
			}
	
			return self::_items($path, $filter, $recurse, $full, $exclude, $excludefilter_string, false);
		}
		public static function files($path, $filter = '.', $recurse = false, $full = false, $exclude = array('.svn', 'CVS', '.DS_Store', '__MACOSX'),
			$excludefilter = array('^\..*', '.*~'), $naturalSort = false) {
	
			$path = rtrim($path, '\\/');
	
			if (!is_dir($path)) {
				return false;
			}
	
			if (count($excludefilter)) {
				$excludefilter_string = '/(' . implode('|', $excludefilter) . ')/';
			} else {
				$excludefilter_string = '';
	
			}
			return self::_items($path, $filter, $recurse, $full, $exclude, $excludefilter_string, true);
		}
		static function _items($path, $filter = '.', $recurse = false, $full = false, $exclude = array('.svn', 'CVS', '.DS_Store', '__MACOSX'), $excludefilter_string='', $findfiles=false) {
			$arr = array();
			// Read the source directory
			if (!($handle = @opendir($path))) {
				return $arr;
			}
	
			while (($file = readdir($handle)) !== false) {
				if ($file != '.' && $file != '..' && !in_array($file, $exclude) && (empty($excludefilter_string) || !preg_match($excludefilter_string, $file))) {
					// Compute the fullpath
					$fullpath = $path . '/' . $file;
	
					// Compute the isDir flag
					$isDir = is_dir($fullpath);
	
					if (($isDir xor $findfiles) && preg_match("/$filter/", $file)) {
						// (fullpath is dir and folders are searched or fullpath is not dir and files are searched) and file matches the filter
						if ($full) {
							// Full path is requested
							$arr[] = $fullpath;
						} else {
							// Filename is requested
							$arr[] = $file;
						}
					}
	
					if ($isDir && $recurse) {
						// Search recursively
						if (is_int($recurse)) {
							// Until depth 0 is reached
							$arr = array_merge($arr, self::_items($fullpath, $filter, $recurse - 1, $full, $exclude, $excludefilter_string, $findfiles));
						} else {
							$arr = array_merge($arr, self::_items($fullpath, $filter, $recurse, $full, $exclude, $excludefilter_string, $findfiles));
						}
					}
				}
			}
	
			closedir($handle);
	
			return $arr;
		}
	}
}

if (!class_exists('fwfmFile')) {
	class fwfmFile {
		public static function upload($src, $dst) {
			$folder = dirname($dst);
			if (!file_exists($folder)) fwfmFolder::create($folder);
			if (file_exists($folder)) return copy($src, $dst);
		}
		public static function getExt($file) {
			$dot = strrpos($file, '.') + 1;
			return substr($file, $dot);
		}
		public static function stripExt($file) {
			return preg_replace('#\.[^.]*$#', '', $file);
		}
		public static function makeSafe($file) {
			$file = rtrim($file, '.');
			$regex = array('#(\.){2,}#', '#[^A-Za-z0-9\.\_\- ]#', '#^\.#');
			return trim(preg_replace($regex, '', $file));
		}
		public static function read($path) {
			if (file_exists($path)) {
				return file_get_contents($path);
			}
		}
		public static function write($path, $data) {
			$folder = dirname($path);
			if (!file_exists($folder)) {
				fwfmFolder::create($folder);
			}
			return file_put_contents($path, $data);
		}
		static function copy($from, $to) {
			$folder = dirname($to);
			if (!file_exists($folder)) fwfmFolder::create($folder);
			if (is_writable($folder)) {
				if (copy($from, $to)) {
					return true;
				} else {
//					$error = error_get_last();
//					die('can\'t copy file '.$from.' to '.$to.' '.$error['message']);
				}
			} else {
//				$error = error_get_last();
//				die('not writeable '.$to.' '.$error['message']);
			}
		}
		static function move($from, $to) {
			$folder = dirname($to);
			if (!file_exists($folder)) fwfmFolder::create($folder);
			if (is_writable($folder)) {
				return rename($from, $to);
			}
		}
		static function delete($path) {
			if (is_file($path) and is_writable($path) and unlink($path)) {
				return true;
			}
		}
	}
}

if (!class_exists('JFilterInput')) {
	class JFilterInput {
		static function sanitazeHtml($el) {
			if (is_numeric($el)) {
				return $el;
			} elseif (is_string($el)) {
				return wp_kses($el, wp_kses_allowed_html('post'));
			} elseif (is_array($el)) {
				foreach ($el as $key=>$val) {
					$el[$key] = JFilterInput::sanitazeHtml($val);
				}
				return $el;
			} elseif (is_object($el)) {
				foreach ($el as $key=>$val) {
					$el->$key = JFilterInput::sanitazeHtml($val);
				}
				return $el;
			}
		}
	}
}
if (!class_exists('fwfmFilterOutput')) {
	class fwfmFilterOutput {
		static function cleanText($text) {
			return esc_html($text);
		}
		static function ampReplace($text) {
			return preg_replace('/(?<!&)&(?!&|#|[\w]+;)/', '&amp;', $text);
		}
		static function stringURLSafe($string) {
			$str = str_replace('-', ' ', $string);
			$str = trim(strtolower($str));
			$str = preg_replace('/(\s|[^A-Za-z0-9\-])+/', '-', $str);
			$str = trim($str, '-');
			return $str;
		}
	}
}

if (!class_exists('fwfmControllerLegacy')) {
	class fwfmControllerLegacy {
		var $name = null,
			$model = null,
			$view = null,
			$input = null,
			$redirect = null,
			$task_map = array();
	
		static function getInstance($name) {
			static $controllers = array();
			if (!isset($controllers[$name])) {
				$path = FWFM_CONTROLLER_DIR.'controller.php';
				if (file_exists($path)) {
					require_once($path);
					$class_name = $name.'Controller';
					if (class_exists($class_name)) {
						$class_name = '\\'.$class_name;
						$controllers[$name] = new $class_name;
					}
				}
			}
			if (isset($controllers[$name])) return $controllers[$name];
		}
		function registerTask($alias_task, $task) {
			$this->task_map[$alias_task] = $task;
		}
		function __construct() {
			$this->name = str_ireplace('controller', '', get_class($this));
			$this->input = fwfmFactory::getApplication()->input;
			$view = $this->input->getString('view');
	
			$this->model = fwfmModelLegacy::getInstance($view, $this->name.'model');
			$this->view = fwfmViewLegacy::getInstance($view, $this->name.'view');
		}
		function getName() {
			return $this->name;
		}
		function getModel() {
			return $this->model;
		}
		function getView() {
			return $this->view;
		}
		function execute($task) {
			$view = $this->input->getString('view');
			$this->model = fwfmModelLegacy::getInstance($view, $this->name.'model');
			$this->view = fwfmViewLegacy::getInstance($view, $this->name.'view');
	
			if (!method_exists($this, $task) and !empty($this->task_map[$task]) and method_exists($this, $this->task_map[$task])) {
				$task = $this->task_map[$task];
			}
			if (method_exists($this, $task)) {
				$this->$task();
			}
		}
		function setRedirect($url, $msg=null) {
			$this->redirect = $url;
			if ($msg) {
				fwfmFactory::getApplication()->enqueueMessage($msg);
			}
		}
		function redirect() {
			if (is_null($this->redirect)) {
				fwfmFactory::getDocument()->render($this->view);
			} else {
				$app = fwfmFactory::getApplication();
				$app->redirect($this->redirect);
			}
		}
		function display() {
			fwfmFactory::getDocument()->render($this->view);
		}
	}
}

if (!class_exists('fwfmPluginHelper')) {
	class fwfmPluginHelper {
		static $plugins = null;
		static function load() {
			if (is_null(static::$plugins)) {
				$db = fwfmFactory::getDBO();
				$db->setQuery('
	SELECT
		extension_id,
		name,
		`folder`,
		`element`,
		params,
		CASE WHEN `element` = \'columns\' THEN 1 ELSE 0 END AS _ordering
	FROM
		`#__extensions`
	WHERE
		`type` = \'plugin\'
		AND
		`enabled` = 1
	ORDER BY
		_ordering');
				static::$plugins = (array)$db->loadObjectList();
			}
			return static::$plugins;
		}
		static function importPlugin($group, $name=null) {
			$plugins = static::load();
			if ($plugins) {
				foreach ($plugins as $plugin) {
					if ($plugin->folder == $group and (!$name or $name == $plugin->element)) {
						$path = FWFM_PATH_SITE.'/plugins/'.$plugin->folder.'/'.$plugin->element.'/'.$plugin->element.'.php';
						if (file_exists($path)) {
							include_once($path);
						}
					}
				}
			}
		}
		public static function getPlugin($type, $plugin = null) {
			$result = array();
			$plugins = static::load();
//print_r($plugins); die();
			// Find the correct plugin(s) to return.
			if (!$plugin) {
				foreach ($plugins as $p) {
					// Is this the right plugin?
					if ($p->type === $type) {
						$result[] = $p;
					}
				}
			}
			else
			{
				foreach ($plugins as $p)
				{
					// Is this plugin in the right group?
					if ($p->folder == $type && $p->element == $plugin) {
						$result = $p;
						break;
					}
				}
			}
	
			return $result;
		}
	}
}

if (!class_exists('fwfmDispatcher')) {
	class fwfmDispatcher {
		static function getInstance() {
			static $instance;
			if (!$instance) {
				$instance = new \fwfmDispatcher;
			}
			return $instance;
		}
		static function trigger($event, $params=array()) {
			$result = array();
			$plugins = fwfmPluginHelper::load();
			foreach ($plugins as $plugin) {
				$class = 'plg'.$plugin->folder.$plugin->element;
				if (class_exists($class)) {
					$subj = '';
					$class = '\\'.$class;
					$obj = new $class($subj, null);
					if (method_exists($obj, $event)) {
						$result[] = call_user_func_array(array($obj, $event), $params);
					}
				}
			}
			return $result;
		}
	}
}

if (!class_exists('fwfmPlugin')) {
	class fwfmPlugin extends fwfmObject {
		var $params;
		function __construct(&$subject, $config = array()) {
			$this->params = new \fwfmRegistry;
			$classname = get_class($this);
			$plugin_name = strtolower(str_ireplace('plgFwFoodMenu', '', $classname));
			$plugin = fwfmPluginHelper::getPlugin('fwfoodmenu', $plugin_name);
			if ($plugin) {
				$this->params->loadString($plugin->params);
			}
		}
	}
}

if (!class_exists('fwfmHttp')) {
	class fwfmHttp {
		static function get($url, $headers=array()) {
			$result = new stdclass;
			$response = wp_remote_get($url, array('headers'=>$headers));
			$result->headers = wp_remote_retrieve_headers($response);
			$result->code = wp_remote_retrieve_response_code($response);
			$result->body = wp_remote_retrieve_body($response);
			return $result;
		}
		static function post($url, $data, $headers=array()) {
			$result = new stdclass;
			$params = array(
				'body'=>$data,
			);
			if ($headers) {
				$params['headers'] = $headers;
			}
			$response = wp_remote_post($url, $params);
			$result->headers = wp_remote_retrieve_headers($response);
			$result->code = wp_remote_retrieve_response_code($response);
			$result->body = wp_remote_retrieve_body($response);
			return $result;
		}
		static function put($url, $data, $headers=array()) {
			$result = new stdclass;
			$http = new WP_Http();
			$params = array(
				'method'=>'put',
				'body'=>$data,
			);
			if ($headers) {
				$params['headers'] = $headers;
			}
			$response = $http->request($url, $params);
			$result->headers = wp_remote_retrieve_headers($response);
			$result->code = wp_remote_retrieve_response_code($response);
			$result->body = wp_remote_retrieve_body($response);
			return $result;
		}
		static function delete($url, $headers=array()) {
			$result = new stdclass;
			$http = new WP_Http();
			$params = array(
				'method'=>'delete',
			);
			if ($headers) {
				$params['headers'] = $headers;
			}
			$response = $http->request($url, $params);
			$result->headers = wp_remote_retrieve_headers($response);
			$result->code = wp_remote_retrieve_response_code($response);
			$result->body = wp_remote_retrieve_body($response);
			return $result;
		}
	}
}

if (!class_exists('fwfmHttpFactory')) {
	class fwfmHttpFactory {
		static function getHttp() {
			return new fwfmHttp();
		}
	}
}

if (!function_exists('jimport')) {
	function jimport() {
	}
}

if (!function_exists('utf8_strcasecmp')) {
	function utf8_strcasecmp($strX, $strY) {
		$strX = utf8_strtolower($strX);
		$strY = utf8_strtolower($strY);
		return strcmp($strX, $strY);
	}
}

if (!function_exists('utf8_strtolower')) {
	function utf8_strtolower($string){
	
		static $UTF8_UPPER_TO_LOWER = NULL;
	
		if ( is_null($UTF8_UPPER_TO_LOWER) ) {
			$UTF8_UPPER_TO_LOWER = array(
		0x0041=>0x0061, 0x03A6=>0x03C6, 0x0162=>0x0163, 0x00C5=>0x00E5, 0x0042=>0x0062,
		0x0139=>0x013A, 0x00C1=>0x00E1, 0x0141=>0x0142, 0x038E=>0x03CD, 0x0100=>0x0101,
		0x0490=>0x0491, 0x0394=>0x03B4, 0x015A=>0x015B, 0x0044=>0x0064, 0x0393=>0x03B3,
		0x00D4=>0x00F4, 0x042A=>0x044A, 0x0419=>0x0439, 0x0112=>0x0113, 0x041C=>0x043C,
		0x015E=>0x015F, 0x0143=>0x0144, 0x00CE=>0x00EE, 0x040E=>0x045E, 0x042F=>0x044F,
		0x039A=>0x03BA, 0x0154=>0x0155, 0x0049=>0x0069, 0x0053=>0x0073, 0x1E1E=>0x1E1F,
		0x0134=>0x0135, 0x0427=>0x0447, 0x03A0=>0x03C0, 0x0418=>0x0438, 0x00D3=>0x00F3,
		0x0420=>0x0440, 0x0404=>0x0454, 0x0415=>0x0435, 0x0429=>0x0449, 0x014A=>0x014B,
		0x0411=>0x0431, 0x0409=>0x0459, 0x1E02=>0x1E03, 0x00D6=>0x00F6, 0x00D9=>0x00F9,
		0x004E=>0x006E, 0x0401=>0x0451, 0x03A4=>0x03C4, 0x0423=>0x0443, 0x015C=>0x015D,
		0x0403=>0x0453, 0x03A8=>0x03C8, 0x0158=>0x0159, 0x0047=>0x0067, 0x00C4=>0x00E4,
		0x0386=>0x03AC, 0x0389=>0x03AE, 0x0166=>0x0167, 0x039E=>0x03BE, 0x0164=>0x0165,
		0x0116=>0x0117, 0x0108=>0x0109, 0x0056=>0x0076, 0x00DE=>0x00FE, 0x0156=>0x0157,
		0x00DA=>0x00FA, 0x1E60=>0x1E61, 0x1E82=>0x1E83, 0x00C2=>0x00E2, 0x0118=>0x0119,
		0x0145=>0x0146, 0x0050=>0x0070, 0x0150=>0x0151, 0x042E=>0x044E, 0x0128=>0x0129,
		0x03A7=>0x03C7, 0x013D=>0x013E, 0x0422=>0x0442, 0x005A=>0x007A, 0x0428=>0x0448,
		0x03A1=>0x03C1, 0x1E80=>0x1E81, 0x016C=>0x016D, 0x00D5=>0x00F5, 0x0055=>0x0075,
		0x0176=>0x0177, 0x00DC=>0x00FC, 0x1E56=>0x1E57, 0x03A3=>0x03C3, 0x041A=>0x043A,
		0x004D=>0x006D, 0x016A=>0x016B, 0x0170=>0x0171, 0x0424=>0x0444, 0x00CC=>0x00EC,
		0x0168=>0x0169, 0x039F=>0x03BF, 0x004B=>0x006B, 0x00D2=>0x00F2, 0x00C0=>0x00E0,
		0x0414=>0x0434, 0x03A9=>0x03C9, 0x1E6A=>0x1E6B, 0x00C3=>0x00E3, 0x042D=>0x044D,
		0x0416=>0x0436, 0x01A0=>0x01A1, 0x010C=>0x010D, 0x011C=>0x011D, 0x00D0=>0x00F0,
		0x013B=>0x013C, 0x040F=>0x045F, 0x040A=>0x045A, 0x00C8=>0x00E8, 0x03A5=>0x03C5,
		0x0046=>0x0066, 0x00DD=>0x00FD, 0x0043=>0x0063, 0x021A=>0x021B, 0x00CA=>0x00EA,
		0x0399=>0x03B9, 0x0179=>0x017A, 0x00CF=>0x00EF, 0x01AF=>0x01B0, 0x0045=>0x0065,
		0x039B=>0x03BB, 0x0398=>0x03B8, 0x039C=>0x03BC, 0x040C=>0x045C, 0x041F=>0x043F,
		0x042C=>0x044C, 0x00DE=>0x00FE, 0x00D0=>0x00F0, 0x1EF2=>0x1EF3, 0x0048=>0x0068,
		0x00CB=>0x00EB, 0x0110=>0x0111, 0x0413=>0x0433, 0x012E=>0x012F, 0x00C6=>0x00E6,
		0x0058=>0x0078, 0x0160=>0x0161, 0x016E=>0x016F, 0x0391=>0x03B1, 0x0407=>0x0457,
		0x0172=>0x0173, 0x0178=>0x00FF, 0x004F=>0x006F, 0x041B=>0x043B, 0x0395=>0x03B5,
		0x0425=>0x0445, 0x0120=>0x0121, 0x017D=>0x017E, 0x017B=>0x017C, 0x0396=>0x03B6,
		0x0392=>0x03B2, 0x0388=>0x03AD, 0x1E84=>0x1E85, 0x0174=>0x0175, 0x0051=>0x0071,
		0x0417=>0x0437, 0x1E0A=>0x1E0B, 0x0147=>0x0148, 0x0104=>0x0105, 0x0408=>0x0458,
		0x014C=>0x014D, 0x00CD=>0x00ED, 0x0059=>0x0079, 0x010A=>0x010B, 0x038F=>0x03CE,
		0x0052=>0x0072, 0x0410=>0x0430, 0x0405=>0x0455, 0x0402=>0x0452, 0x0126=>0x0127,
		0x0136=>0x0137, 0x012A=>0x012B, 0x038A=>0x03AF, 0x042B=>0x044B, 0x004C=>0x006C,
		0x0397=>0x03B7, 0x0124=>0x0125, 0x0218=>0x0219, 0x00DB=>0x00FB, 0x011E=>0x011F,
		0x041E=>0x043E, 0x1E40=>0x1E41, 0x039D=>0x03BD, 0x0106=>0x0107, 0x03AB=>0x03CB,
		0x0426=>0x0446, 0x00DE=>0x00FE, 0x00C7=>0x00E7, 0x03AA=>0x03CA, 0x0421=>0x0441,
		0x0412=>0x0432, 0x010E=>0x010F, 0x00D8=>0x00F8, 0x0057=>0x0077, 0x011A=>0x011B,
		0x0054=>0x0074, 0x004A=>0x006A, 0x040B=>0x045B, 0x0406=>0x0456, 0x0102=>0x0103,
		0x039B=>0x03BB, 0x00D1=>0x00F1, 0x041D=>0x043D, 0x038C=>0x03CC, 0x00C9=>0x00E9,
		0x00D0=>0x00F0, 0x0407=>0x0457, 0x0122=>0x0123,
				);
		}
	
		$uni = utf8_to_unicode($string);
	
		if ( !$uni ) {
			return FALSE;
		}
	
		$cnt = count($uni);
		for ($i=0; $i < $cnt; $i++){
			if ( isset($UTF8_UPPER_TO_LOWER[$uni[$i]]) ) {
				$uni[$i] = $UTF8_UPPER_TO_LOWER[$uni[$i]];
			}
		}
	
		return utf8_from_unicode($uni);
	}
}

if (!function_exists('utf8_to_unicode')) {
	function utf8_to_unicode($str) {
		$mState = 0;     // cached expected number of octets after the current octet
						 // until the beginning of the next UTF8 character sequence
		$mUcs4  = 0;     // cached Unicode character
		$mBytes = 1;     // cached expected number of octets in the current sequence
	
		$out = array();
	
		$len = strlen($str);
	
		for($i = 0; $i < $len; $i++) {
	
			$in = ord($str[$i]);
	
			if ( $mState == 0) {
	
				// When mState is zero we expect either a US-ASCII character or a
				// multi-octet sequence.
				if (0 == (0x80 & ($in))) {
					// US-ASCII, pass straight through.
					$out[] = $in;
					$mBytes = 1;
	
				} else if (0xC0 == (0xE0 & ($in))) {
					// First octet of 2 octet sequence
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 0x1F) << 6;
					$mState = 1;
					$mBytes = 2;
	
				} else if (0xE0 == (0xF0 & ($in))) {
					// First octet of 3 octet sequence
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 0x0F) << 12;
					$mState = 2;
					$mBytes = 3;
	
				} else if (0xF0 == (0xF8 & ($in))) {
					// First octet of 4 octet sequence
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 0x07) << 18;
					$mState = 3;
					$mBytes = 4;
	
				} else if (0xF8 == (0xFC & ($in))) {
					/* First octet of 5 octet sequence.
					*
					* This is illegal because the encoded codepoint must be either
					* (a) not the shortest form or
					* (b) outside the Unicode range of 0-0x10FFFF.
					* Rather than trying to resynchronize, we will carry on until the end
					* of the sequence and let the later error handling code catch it.
					*/
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 0x03) << 24;
					$mState = 4;
					$mBytes = 5;
	
				} else if (0xFC == (0xFE & ($in))) {
					// First octet of 6 octet sequence, see comments for 5 octet sequence.
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 1) << 30;
					$mState = 5;
					$mBytes = 6;
	
				} else {
					/* Current octet is neither in the US-ASCII range nor a legal first
					 * octet of a multi-octet sequence.
					 */
					trigger_error(
							'utf8_to_unicode: Illegal sequence identifier '.
								'in UTF-8 at byte '.$i,
							E_USER_WARNING
						);
					return FALSE;
	
				}
	
			} else {
	
				// When mState is non-zero, we expect a continuation of the multi-octet
				// sequence
				if (0x80 == (0xC0 & ($in))) {
	
					// Legal continuation.
					$shift = ($mState - 1) * 6;
					$tmp = $in;
					$tmp = ($tmp & 0x0000003F) << $shift;
					$mUcs4 |= $tmp;
	
					/**
					* End of the multi-octet sequence. mUcs4 now contains the final
					* Unicode codepoint to be output
					*/
					if (0 == --$mState) {
	
						/*
						* Check for illegal sequences and codepoints.
						*/
						// From Unicode 3.1, non-shortest form is illegal
						if (((2 == $mBytes) && ($mUcs4 < 0x0080)) ||
							((3 == $mBytes) && ($mUcs4 < 0x0800)) ||
							((4 == $mBytes) && ($mUcs4 < 0x10000)) ||
							(4 < $mBytes) ||
							// From Unicode 3.2, surrogate characters are illegal
							(($mUcs4 & 0xFFFFF800) == 0xD800) ||
							// Codepoints outside the Unicode range are illegal
							($mUcs4 > 0x10FFFF)) {
	
							trigger_error(
									'utf8_to_unicode: Illegal sequence or codepoint '.
										'in UTF-8 at byte '.$i,
									E_USER_WARNING
								);
	
							return FALSE;
	
						}
	
						if (0xFEFF != $mUcs4) {
							// BOM is legal but we don't want to output it
							$out[] = $mUcs4;
						}
	
						//initialize UTF8 cache
						$mState = 0;
						$mUcs4  = 0;
						$mBytes = 1;
					}
	
				} else {
					/**
					*((0xC0 & (*in) != 0x80) && (mState != 0))
					* Incomplete multi-octet sequence.
					*/
					trigger_error(
							'utf8_to_unicode: Incomplete multi-octet '.
							'   sequence in UTF-8 at byte '.$i,
							E_USER_WARNING
						);
	
					return FALSE;
				}
			}
		}
		return $out;
	}
}

if (!function_exists('utf8_from_unicode')) {
	function utf8_from_unicode($arr) {
		ob_start();
	
		foreach (array_keys($arr) as $k) {
	
			# ASCII range (including control chars)
			if ( ($arr[$k] >= 0) && ($arr[$k] <= 0x007f) ) {
	
				echo chr($arr[$k]);
	
			# 2 byte sequence
			} else if ($arr[$k] <= 0x07ff) {
	
				echo chr(0xc0 | ($arr[$k] >> 6));
				echo chr(0x80 | ($arr[$k] & 0x003f));
	
			# Byte order mark (skip)
			} else if($arr[$k] == 0xFEFF) {
	
				// nop -- zap the BOM
	
			# Test for illegal surrogates
			} else if ($arr[$k] >= 0xD800 && $arr[$k] <= 0xDFFF) {
	
				// found a surrogate
				trigger_error(
					'utf8_from_unicode: Illegal surrogate '.
						'at index: '.$k.', value: '.$arr[$k],
					E_USER_WARNING
					);
	
				return FALSE;
	
			# 3 byte sequence
			} else if ($arr[$k] <= 0xffff) {
	
				echo chr(0xe0 | ($arr[$k] >> 12));
				echo chr(0x80 | (($arr[$k] >> 6) & 0x003f));
				echo chr(0x80 | ($arr[$k] & 0x003f));
	
			# 4 byte sequence
			} else if ($arr[$k] <= 0x10ffff) {
	
				echo chr(0xf0 | ($arr[$k] >> 18));
				echo chr(0x80 | (($arr[$k] >> 12) & 0x3f));
				echo chr(0x80 | (($arr[$k] >> 6) & 0x3f));
				echo chr(0x80 | ($arr[$k] & 0x3f));
	
			} else {
	
				trigger_error(
					'utf8_from_unicode: Codepoint out of Unicode range '.
						'at index: '.$k.', value: '.$arr[$k],
					E_USER_WARNING
					);
	
				// out of range
				return FALSE;
			}
		}
	
		$result = ob_get_contents();
		ob_end_clean();
		return $result;
	}
}

if (!function_exists('url_slug')) {
	/**
	 * Create a web friendly URL slug from a string.
	 * 
	 * Although supported, transliteration is discouraged because
	 *     1) most web browsers support UTF-8 characters in URLs
	 *     2) transliteration causes a loss of information
	 *
	 * @author Sean Murphy <sean@iamseanmurphy.com>
	 * @copyright Copyright 2012 Sean Murphy. All rights reserved.
	 * @license http://creativecommons.org/publicdomain/zero/1.0/
	 *
	 * @param string $str
	 * @param array $options
	 * @return string
	 */
	function url_slug($str, $options = array()) {
		// Make sure string is in UTF-8 and strip invalid UTF-8 characters
		$str = mb_convert_encoding((string)$str, 'UTF-8', mb_list_encodings());
		
		$defaults = array(
			'delimiter' => '-',
			'limit' => null,
			'lowercase' => true,
			'replacements' => array(),
			'transliterate' => false,
		);
		
		// Merge options
		$options = array_merge($defaults, $options);
		
		$char_map = array(
			// Latin
			'À' => 'A', 'Á' => 'A', 'Â' => 'A', 'Ã' => 'A', 'Ä' => 'A', 'Å' => 'A', 'Æ' => 'AE', 'Ç' => 'C', 
			'È' => 'E', 'É' => 'E', 'Ê' => 'E', 'Ë' => 'E', 'Ì' => 'I', 'Í' => 'I', 'Î' => 'I', 'Ï' => 'I', 
			'Ð' => 'D', 'Ñ' => 'N', 'Ò' => 'O', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ö' => 'O', 'Ő' => 'O', 
			'Ø' => 'O', 'Ù' => 'U', 'Ú' => 'U', 'Û' => 'U', 'Ü' => 'U', 'Ű' => 'U', 'Ý' => 'Y', 'Þ' => 'TH', 
			'ß' => 'ss', 
			'à' => 'a', 'á' => 'a', 'â' => 'a', 'ã' => 'a', 'ä' => 'a', 'å' => 'a', 'æ' => 'ae', 'ç' => 'c', 
			'è' => 'e', 'é' => 'e', 'ê' => 'e', 'ë' => 'e', 'ì' => 'i', 'í' => 'i', 'î' => 'i', 'ï' => 'i', 
			'ð' => 'd', 'ñ' => 'n', 'ò' => 'o', 'ó' => 'o', 'ô' => 'o', 'õ' => 'o', 'ö' => 'o', 'ő' => 'o', 
			'ø' => 'o', 'ù' => 'u', 'ú' => 'u', 'û' => 'u', 'ü' => 'u', 'ű' => 'u', 'ý' => 'y', 'þ' => 'th', 
			'ÿ' => 'y',
	
			// Latin symbols
			'©' => '(c)',
	
			// Greek
			'Α' => 'A', 'Β' => 'B', 'Γ' => 'G', 'Δ' => 'D', 'Ε' => 'E', 'Ζ' => 'Z', 'Η' => 'H', 'Θ' => '8',
			'Ι' => 'I', 'Κ' => 'K', 'Λ' => 'L', 'Μ' => 'M', 'Ν' => 'N', 'Ξ' => '3', 'Ο' => 'O', 'Π' => 'P',
			'Ρ' => 'R', 'Σ' => 'S', 'Τ' => 'T', 'Υ' => 'Y', 'Φ' => 'F', 'Χ' => 'X', 'Ψ' => 'PS', 'Ω' => 'W',
			'Ά' => 'A', 'Έ' => 'E', 'Ί' => 'I', 'Ό' => 'O', 'Ύ' => 'Y', 'Ή' => 'H', 'Ώ' => 'W', 'Ϊ' => 'I',
			'Ϋ' => 'Y',
			'α' => 'a', 'β' => 'b', 'γ' => 'g', 'δ' => 'd', 'ε' => 'e', 'ζ' => 'z', 'η' => 'h', 'θ' => '8',
			'ι' => 'i', 'κ' => 'k', 'λ' => 'l', 'μ' => 'm', 'ν' => 'n', 'ξ' => '3', 'ο' => 'o', 'π' => 'p',
			'ρ' => 'r', 'σ' => 's', 'τ' => 't', 'υ' => 'y', 'φ' => 'f', 'χ' => 'x', 'ψ' => 'ps', 'ω' => 'w',
			'ά' => 'a', 'έ' => 'e', 'ί' => 'i', 'ό' => 'o', 'ύ' => 'y', 'ή' => 'h', 'ώ' => 'w', 'ς' => 's',
			'ϊ' => 'i', 'ΰ' => 'y', 'ϋ' => 'y', 'ΐ' => 'i',
	
			// Turkish
			'Ş' => 'S', 'İ' => 'I', 'Ç' => 'C', 'Ü' => 'U', 'Ö' => 'O', 'Ğ' => 'G',
			'ş' => 's', 'ı' => 'i', 'ç' => 'c', 'ü' => 'u', 'ö' => 'o', 'ğ' => 'g', 
	
			// Russian
			'А' => 'A', 'Б' => 'B', 'В' => 'V', 'Г' => 'G', 'Д' => 'D', 'Е' => 'E', 'Ё' => 'Yo', 'Ж' => 'Zh',
			'З' => 'Z', 'И' => 'I', 'Й' => 'J', 'К' => 'K', 'Л' => 'L', 'М' => 'M', 'Н' => 'N', 'О' => 'O',
			'П' => 'P', 'Р' => 'R', 'С' => 'S', 'Т' => 'T', 'У' => 'U', 'Ф' => 'F', 'Х' => 'H', 'Ц' => 'C',
			'Ч' => 'Ch', 'Ш' => 'Sh', 'Щ' => 'Sh', 'Ъ' => '', 'Ы' => 'Y', 'Ь' => '', 'Э' => 'E', 'Ю' => 'Yu',
			'Я' => 'Ya',
			'а' => 'a', 'б' => 'b', 'в' => 'v', 'г' => 'g', 'д' => 'd', 'е' => 'e', 'ё' => 'yo', 'ж' => 'zh',
			'з' => 'z', 'и' => 'i', 'й' => 'j', 'к' => 'k', 'л' => 'l', 'м' => 'm', 'н' => 'n', 'о' => 'o',
			'п' => 'p', 'р' => 'r', 'с' => 's', 'т' => 't', 'у' => 'u', 'ф' => 'f', 'х' => 'h', 'ц' => 'c',
			'ч' => 'ch', 'ш' => 'sh', 'щ' => 'sh', 'ъ' => '', 'ы' => 'y', 'ь' => '', 'э' => 'e', 'ю' => 'yu',
			'я' => 'ya',
	
			// Ukrainian
			'Є' => 'Ye', 'І' => 'I', 'Ї' => 'Yi', 'Ґ' => 'G',
			'є' => 'ye', 'і' => 'i', 'ї' => 'yi', 'ґ' => 'g',
	
			// Czech
			'Č' => 'C', 'Ď' => 'D', 'Ě' => 'E', 'Ň' => 'N', 'Ř' => 'R', 'Š' => 'S', 'Ť' => 'T', 'Ů' => 'U', 
			'Ž' => 'Z', 
			'č' => 'c', 'ď' => 'd', 'ě' => 'e', 'ň' => 'n', 'ř' => 'r', 'š' => 's', 'ť' => 't', 'ů' => 'u',
			'ž' => 'z', 
	
			// Polish
			'Ą' => 'A', 'Ć' => 'C', 'Ę' => 'e', 'Ł' => 'L', 'Ń' => 'N', 'Ó' => 'o', 'Ś' => 'S', 'Ź' => 'Z', 
			'Ż' => 'Z', 
			'ą' => 'a', 'ć' => 'c', 'ę' => 'e', 'ł' => 'l', 'ń' => 'n', 'ó' => 'o', 'ś' => 's', 'ź' => 'z',
			'ż' => 'z',
	
			// Latvian
			'Ā' => 'A', 'Č' => 'C', 'Ē' => 'E', 'Ģ' => 'G', 'Ī' => 'i', 'Ķ' => 'k', 'Ļ' => 'L', 'Ņ' => 'N', 
			'Š' => 'S', 'Ū' => 'u', 'Ž' => 'Z',
			'ā' => 'a', 'č' => 'c', 'ē' => 'e', 'ģ' => 'g', 'ī' => 'i', 'ķ' => 'k', 'ļ' => 'l', 'ņ' => 'n',
			'š' => 's', 'ū' => 'u', 'ž' => 'z'
		);
		
		// Make custom replacements
		$str = preg_replace(array_keys($options['replacements']), $options['replacements'], $str);
		
		// Transliterate characters to ASCII
		if ($options['transliterate']) {
			$str = str_replace(array_keys($char_map), $char_map, $str);
		}
		
		// Replace non-alphanumeric characters with our delimiter
		$str = preg_replace('/[^\p{L}\p{Nd}]+/u', $options['delimiter'], $str);
		
		// Remove duplicate delimiters
		$str = preg_replace('/(' . preg_quote($options['delimiter'], '/') . '){2,}/', '$1', $str);
		
		// Truncate slug to max. characters
		$str = mb_substr($str, 0, ($options['limit'] ? $options['limit'] : mb_strlen($str, 'UTF-8')), 'UTF-8');
		
		// Remove delimiter from ends
		$str = trim($str, $options['delimiter']);
		
		return $options['lowercase'] ? mb_strtolower($str, 'UTF-8') : $str;
	}
}

define('FWFM_CONTROLLER_DIR', FWFM_DIR.(fwfmFactory::getApplication()->isAdmin()?'admin/':'site/'));
define('FWFM_PATH_SITE', rtrim(FWFM_DIR, '\\/'));
define('FWFM_PATH_BASE', FWFM_PATH_SITE);
define('FWFM_PATH_ADMINISTRATOR', FWFM_PATH_SITE);
define('FWFM_PATH_COMPONENT', rtrim(FWFM_CONTROLLER_DIR, '\\/'));
define('FWFM_PATH_COMPONENT_SITE', FWFM_DIR.'site');
define('FWFM_PATH_COMPONENT_ADMINISTRATOR', FWFM_DIR.'admin');
define('FWFM_COMPONENT_SITE', FWFM_DIR.'site');
define('FWFM_COMPONENT_ADMINISTRATOR', FWFM_DIR.'admin');
define('FWFM_LANGUAGE_SITE', FWFM_COMPONENT_SITE);
define('FWFM_LANGUAGE_ADMINISTRATOR', FWFM_COMPONENT_ADMINISTRATOR);
define('FWFM_ASSETS_URI', plugin_dir_url('fw-food-menu/site/assets/index.html'));
define('FWFM_ADMIN_ASSETS_URI', plugin_dir_url('fw-food-menu/admin/assets/index.html'));
